import os, sys
sys.path.append(os.path.dirname(__file__)[:-6][:-4][:-13])
from cognitivegeo.src.segpy.header import FormatMeta, field
from cognitivegeo.src.segpy.types import Int32, Int16


class TraceHeaderRev0(metaclass=FormatMeta):

    START_OFFSET_IN_BYTES = 1
    LENGTH_IN_BYTES = 240

    line_sequence_num = field(
        Int32, offset=1, default=0, documentation=
        "Trace sequence number within line — Numbers continue to increase if the same line "
        "continues across multiple SEG Y files. Highly recommended for all types of data.")

    file_sequence_num = field(
        Int32, offset=5, default=0, documentation=
        "Trace sequence number within SEG Y file — Each file starts with trace sequence one.")

    field_record_num = field(
        Int32, offset=9, default=0, documentation=
        "Original field record number. Highly recommended for all types of data.")

    trace_num = field(
        Int32, offset=13, default=0, documentation=
        "Trace number within the original field record. Highly recommended for all types of data.")

    energy_source_point_num = field(
        Int32, offset=17, default=0, documentation=
        "Energy source point number — Used when more than one record occurs at the same "
        "effective surface location. It is recommended that the new entry defined in Trace "
        "Header bytes 197-202 be used for shotpoint number.")

    ensemble_num = field(
        Int32, offset=21, default=0, documentation=
        "Ensemble number (i.e. CDP , CMP , CRP , etc)")

    ensemble_trace_num = field(
        Int32, offset=25, default=0, documentation=
        "Trace number within the ensemble — Each ensemble starts with trace number one.")

    trace_identification_code = field(
        Int16, offset=29, default=0, documentation=
        "Trace identification code")

    num_vertically_summed_traces = field(
        Int16, offset=31, default=1, documentation=
        "Number of vertically summed traces yielding this trace. (1 is one trace, 2 is two summed traces, etc.)")

    num_horizontally_stacked_traces = field(
        Int16, offset=33, default=1, documentation=
        "Number of horizontally stacked traces yielding this trace. (1 is one trace, 2 is two stacked traces, etc.)")

    data_use = field(
        Int16, offset=35, default=1, documentation=
        "Data use: 1 = Production, 2 = Test")

    source_receiver_offset = field(
        Int32, offset=37, default=0, documentation=
        "Distance from center of the source point to the center of the receiver group (negative if opposite to "
        "direction in which line is shot).")

    receiver_group_elevation = field(
        Int32, offset=41, default=0, documentation=
        "Receiver group elevation (all elevations above the Vertical datum are positive and below are negative). The "
        "elevation_scalar applies to this value.")

    surface_elevation_at_source = field(
        Int32, offset=45, default=0, documentation=
        "Surface elevation at source. The elevation_scalar applies to this value.")

    source_depth_below_surface = field(
        Int32, offset=49, default=0, documentation=
        "Source depth below surface (a positive number). The elevation_scalar applies to this value.")

    datum_elevation_at_receiver_group = field(
        Int32, offset=53, default=0, documentation=
        "Source depth below surface (a positive number). The elevation_scalar applies to this value.")

    datum_elevation_at_source = field(
        Int32, offset=57, default=0, documentation=
        "Datum elevation at source. The elevation_scalar applies to this value.")

    water_depth_at_source = field(
        Int32, offset=61, default=0, documentation=
        "Water depth at source. The elevation_scalar applies to this value.")

    water_depth_at_group = field(
        Int32, offset=65, default=0, documentation=
        "Water depth at group. The elevation_scalar applies to this value."
    )

    elevation_scalar = field(
        Int16, offset=69, default=1, documentation=
        "Scalar to be applied to the elevations and depths specified in: receiver_group_elevation, "
        "surface_elevation_at_source, source_depth_below_surface, datum_elevation_at_receiver_group, "
        "datum_elevation_at_source, water_depth_at_source and water_depth_at_group, to give the real value. "
        "Scalar = 1, +10, +100, +1000, or +10,000. If positive, scalar is used as a multiplier; if negative, scalar is "
        "used as a divisor."
    )

    xy_scalar = field(
        Int16, offset=71, default=1, documentation=
        "Scalar to be applied to all coordinates specified in source_x, source_y, group_x, group_y, cdp_x and cdp_y to "
        "give the real value. Scalar = 1, +10, +100, +1000, or +10,000. If positive, scalar is used as a multiplier; "
        "if negative, scalar is used as divisor."
    )

    source_x = field(
        Int32, offset=73, default=0, documentation=
        "Source coordinate - X. The xy_scalar applies to this value. The coordinate reference system should be "
        "identified through an extended header Location Data stanza. If the coordinate units are in seconds of arc, "
        "decimal degrees or DMS, the X values represent longitude. A positive value designates east of Greenwich "
        "Meridian and a negative value designates west."
    )

    source_y = field(
        Int32, offset=77, default=0, documentation=
        "Source coordinate - Y. The xy_scalar applies to this value. The coordinate reference system should be "
        "identified through an extended header Location Data stanza. If the coordinate units are in seconds of arc, "
        "decimal degrees or DMS, the Y values represent latitude. A positive value designates north of the equator and "
        "a negative value designates south."
    )

    group_x = field(
        Int32, offset=81, default=0, documentation=
        "Group coordinate - X. The xy_scalar applies to this value. The coordinate reference system should be "
        "identified through an extended header Location Data stanza. If the coordinate units are in seconds of arc, "
        "decimal degrees or DMS, the X values represent longitude. A positive value designates east of Greenwich "
        "Meridian and a negative value designates west."
    )

    group_y = field(
        Int32, offset=85, default=0, documentation=
        "Source coordinate - Y. The xy_scalar applies to this value. The coordinate reference system should be "
        "identified through an extended header Location Data stanza. If the coordinate units are in seconds of arc, "
        "decimal degrees or DMS, the Y values represent latitude. A positive value designates north of the equator and "
        "a negative value designates south."
    )

    coordinate_units = field(
        Int16, offset=89, default=0, documentation=
        "Coordinate units: 1 = Length (meters or feet), 2 = Seconds of arc, 3 = Decimal degrees, 4 = Degrees, minutes, "
        "seconds (DMS). Note: To encode ±DDDMMSS bytes this value equals ±DDD*104 + MM*102 + SS with xy_scalar set to "
        "1; To encode ±DDDMMSS.ss this value equals ±DDD*106 + MM*104 + SS*102 with xy_scalar set to -100."
    )

    weathering_velocity = field(
        Int16, offset=91, default=0, documentation=
        "Weathering velocity. (ft/s or m/s as specified in Binary File Header bytes 3255- 3256)"  # TODO
    )

    subweathering_velocity = field(
        Int16, offset=93, default=0, documentation=
        "Subweathering velocity. (ft/s or m/s as specified in Binary File Header bytes 3255-3256)"  # TODO
    )

    uphole_time_at_source = field(
        Int16, offset=95, default=0, documentation=
        "Uphole time at source in milliseconds. The time_scalar applies to this value."
    )

    uphole_time_at_group = field(
        Int16, offset=97, default=0, documentation=
        "Uphole time at group in milliseconds. The time_scalar applies to this value."
    )

    source_static_correction = field(
        Int16, offset=99, default=0, documentation=
        "Source static correction in milliseconds. The time_scalar applies to this value."
    )

    group_static_correction = field(
        Int16, offset=101, default=0, documentation=
        "Group static correction in milliseconds. The time_scalar applies to this value."
    )

    total_static = field(
        Int16, offset=103, default=0, documentation=
        "Total static applied in milliseconds. (Zero if no static has been applied). "
        "The time_scalar applies to this value."
    )

    lag_time_a = field(
        Int16, offset=105, default=0, documentation=
        "Lag time A. Time in milliseconds between end of 240-byte trace identification header and time break. "
        "The value is positive if time break occurs after the end of header; negative if time break occurs before the "
        "end of header. Time break is defined as the initiation pulse that may be recorded on an auxiliary trace or as "
        "otherwise specified by the recording system. The time_scalar applies to this value."
    )

    lag_time_b = field(
        Int16, offset=107, default=0, documentation=
        "Lag Time B. Time in milliseconds between time break and the initiation time of the energy source. May be "
        "positive or negative. The time_scalar applies to this value."
    )

    delay_recording_time = field(
        Int16, offset=109, default=0, documentation=
        "Delay recording time — Time in milliseconds between initiation time of energy source and the time when "
        "recording of data samples begins. In SEG Y rev 0 this entry was intended for deep-water work if data "
        "recording does not start at zero time. The entry can be negative to accommodate negative start times (i.e. "
        "data recorded before time zero, presumably as a result of static application to the data trace). If a "
        "non-zero value (negative or positive) is recorded in this entry, a comment to that effect should appear in "
        "the Textual File Header. The time_scalar applies to this value."
    )

    mute_start_time = field(
        Int16, offset=111, default=0, documentation=
        "Mute time — Start time in milliseconds. The time_scalar applies to this value."
    )

    mute_end_time = field(
        Int16, offset=113, default=0, documentation=
        "Mute time — End time in milliseconds.  The time_scalar applies to this value."
    )

    num_samples = field(
        Int16, offset=115, default=0, documentation=
        "Number of samples in this trace. Highly recommended for all types of data."
    )

    sample_interval = field(
        Int16, offset=117, default=0, documentation=
        "Sample interval in microseconds (μs) for this trace. The number of bytes in a trace record must be consistent "
        "with the number of samples written in the trace header. This is important for all recording media; but it is "
        "particularly crucial for the correct processing of SEG Y data in disk files (see Appendix C). "
        "If the fixed length trace flag in bytes 3503-3504 of the Binary File Header is set, the sample interval and "   #TODO: field names instead of byte offsets
        "number of samples in every trace in the SEG Y file must be the same as the values recorded in the Binary File "
        "Header. If the fixed length trace flag is not set, the sample interval and number of samples may vary from "
        "trace to trace. Highly recommended for all types of data."
    )

    gain_type_of_field_instruments = field(
        Int16, offset=119, default=0, documentation=
        "Gain type of field instruments. 1 = fixed, 2 = binary, 3 = floating point, 4 ... N = optional use."
    )

    instrument_gain_constant = field(
        Int16, offset=121, default=0, documentation=
        "Instrument gain constant (dB)."
    )

    instrument_initial_gain = field(
        Int16, offset=123, default=0, documentation=
        "Instrument early or initial gain (dB)."
    )

    correlated = field(
        Int16, offset=125, default=1, documentation=
        "Correlated: 1 = no, 2 = yes."
    )

    sweep_frequency_at_start = field(
        Int16, offset=127, default=0, documentation=
        "Sweep frequency at start (Hz)."
    )

    sweep_frequency_at_end = field(
        Int16, offset=129, default=0, documentation=
        "Sweep frequency at end (Hz)."
    )

    sweep_length = field(
        Int16, offset=131, default=0, documentation=
        "Sweep length in milliseconds."
    )

    sweep_type = field(
        Int16, offset=133, default=0, documentation=
        "Sweep type: 1 = linear, 2 = parabolic, 3 = exponential 4 = other."
    )
    sweep_trace_taper_length_at_start = field(
        Int16, offset=135, default=0, documentation=
        "Sweep trace taper length at start in milliseconds."
    )

    sweep_trace_taper_length_at_end = field(
        Int16, offset=137, default=0, documentation=
        "Sweep trace taper length at end in milliseconds."
    )

    taper_type = field(
        Int16, offset=139, default=0, documentation=
        "Taper type: 1 = linear, 2 = cos2, 3 = other"
    )

    alias_filter_frequency = field(
        Int16, offset=141, default=0, documentation=
        "Alias filter frequency (Hz), if used."
    )

    alias_filter_slope = field(
        Int16, offset=143, default=0, documentation=
        "Alias filter slope (dB/octave)."
    )

    notch_filter_frequency = field(
        Int16, offset=145, default=0, documentation=
        "Notch filter frequency (Hz), if used."
    )

    notch_filter_slope = field(
        Int16, offset=147, default=0, documentation=
        "Notch filter slope (dB/octave)."
    )

    low_cut_frequency = field(
        Int16, offset=149, default=0, documentation=
        "Low-cut frequency (Hz), if used."
    )

    high_cut_frequency = field(
        Int16, offset=151, default=0, documentation=
        "High-cut frequency (Hz), if used."
    )

    low_cut_slope = field(
        Int16, offset=153, default=0, documentation=
        "Low-cut slope (dB/octave)."
    )

    high_cut_slope = field(
        Int16, offset=155, default=0, documentation=
        "High-cut slope (dB/octave)."
    )

    year_recorded = field(
        Int16, offset=155, default=0, documentation=
        "Year data recorded. The 1975 standard is unclear as to whether this should be recorded as a 2-digit or a "
        "4-digit year and both have been used. For SEG Y revisions beyond rev 0, the year should be recorded as the "
        "complete 4-digit Gregorian calendar year (i.e. the year 2001 should be recorded as 2001 in base 10 "
        "(7D1 in base16))."
    )

    day_of_year = field(
        Int16, offset=159, default=0, documentation=
        "Day of year. (Julian day for GMT and UTC time basis)."
    )

    hour_of_day = field(
        Int16, offset=161, default=0, documentation=
        "Hour of day. (24 hour clock)."
    )

    minute_of_hour = field(
        Int16, offset=163, default=0, documentation=
        "Minute of hour."
    )

    second_of_minute = field(
        Int16, offset=165, default=0, documentation=
        "Second of minute."
    )

    time_basis_code = field(
        Int16, offset=167, default=0, documentation=
        "Time basis code. 1 = Local, 2 = GMT (Greenwich Mean Time), 3 = Other, should be explained in a user defined "
        "stanza in the Extended Textual File Header, 4 = UTC (Coordinated Universal Time)."
    )

    trace_weighting_factor = field(
        Int16, offset=169, default=0, documentation=
        "Trace weighting factor. Defined as 2**-N volts for the least significant bit. (N = 0, 1, ..., 32767)."
    )

    geophone_group_num_roll_switch_position_one = field(
        Int16, offset=171, default=0, documentation=
        "Geophone group number of roll switch position one."
    )

    geophone_group_num_first_trace_original_field = field(
        Int16, offset=173, default=0, documentation=
        "Geophone group number of trace number one within original field record."
    )

    geophone_group_num_last_trace_original_field = field(
        Int16, offset=175, default=0, documentation=
        "Geophone group number of last trace within original field record."
    )

    gap_size = field(
        Int16, offset=177, default=0, documentation=
        "Gap size. (Total number of groups dropped)."
    )

    over_travel = field(
        Int16, offset=179, default=0, documentation=
        "Over travel associated with taper at beginning or end of line. 1 = down (or behind) 2 = up (or ahead)"
    )


class TraceHeaderRev1(TraceHeaderRev0, metaclass=FormatMeta):

    cdp_x = field(
        Int32, offset=181, default=0, documentation=
        "X coordinate of ensemble (CDP) position of this trace. "
        "The coordinate reference system should be identified through an extended header Location Data stanza. "
        "The xy_scalar field applies to this value."
    )

    cdp_y = field(
        Int32, offset=185, default=0, documentation=
        "Y coordinate of ensemble (CDP) position of this trace. "
        "The coordinate reference system should be identified through an extended header Location Data stanza. "
        "The xy_scalar field applies to this value."
    )

    inline_number = field(
        Int32, offset=189, default=0, documentation=
        "In-line number for 3-D poststack data. If one in-line per SEG Y file is "
        "being recorded, this value should be the same for all traces in the file and the same value will be recorded "
        "in bytes 3205-3208 of the Binary File Header."  # TODO: replace bytes with field name
    )

    crossline_number = field(
        Int32, offset=193, default=0, documentation=
        "Cross-line number for 3-D poststack data. This will typically be the same "
        "value as the ensemble (CDP) number in Trace Header ensemble_num field, but this does not have to be the case."
    )

    shotpoint_number = field(
        Int32, offset=197, default=0, documentation=
        "Shotpoint number. This is probably only applicable to 2-D poststack data. Note that it is assumed that the "
        "shotpoint number refers to the source location nearest to the ensemble (CDP) location for a particular trace. "
        "If this is not the case, there should be a comment in the Textual File Header explaining what the shotpoint "
        "number actually refers to."
    )

    shotpoint_scalar = field(
        Int16, offset=201, default=0, documentation=
        "Scalar to be applied to the shotpoint number in trace header field shotpoint_number to give the real value. "
        "If positive, scalar is used as a multiplier; if negative as a divisor; if zero the shotpoint number is not "
        "scaled (i.e. it is an integer. A typical value will be -10, allowing shotpoint numbers with one decimal digit "
        "to the right of the decimal point)."
    )

    trace_unit = field(
        Int16, offset=203, default=0, documentation=
        "Trace value measurement unit: "
        "-1 = Other (should be described in Data Sample Measurement Units Stanza)"
        "0 = Unknown, "
        "1 = Pascal (Pa), "
        "2 = Volts (v), "
        "3 = Millivolts (mV), "
        "4 = Amperes (A), "
        "5 = Meters (m), "
        "6 = Meters per second (m/s), "
        "7 = Meters per second squared (m/s^2), "
        "8 = Newton (N), "
        "9 = Watt (W)"
    )

    transduction_constant_mantissa = field(
        Int32, offset=205, default=0, documentation=
        "Transduction Constant mantissa. The mantissa of the multiplicative constant used to convert the Data Trace "
        "samples to the Transduction Units (specified in Trace Header transduction_units field)."
        "The mantissa of the constant is encoded as a four-byte, two's complement integer. The constant value is given"
        "by transduction_constant_mantissa * 10**transduction_constant_exponent."
    )

    transduction_constant_exponent = field(
        Int16, offset=209, default=0, documentation=
        "Transduction Constant exponent. The base 10 exponent of the multiplicative constant used to convert the Data "
        "Trace samples to the Transduction Units (specified in Trace Header transduction_units field). The constant "
        "value is given by transduction_constant_mantissa * 10**transduction_constant_exponent."
    )

    transduction_units = field(
        Int16, offset=211, default=0, documentation=
        "Transduction Units. The unit of measurement of the Data Trace samples after they have been multiplied by the "
        "Transduction Constant specified in Trace Header fields transduction_constant_mantissa and "
        "transduction_constant_exponent. "
        "-1 = Other (should be described in Data Sample Measurement Unit stanza), "
        "0 = Unknown, "
        "1 = Pascal (Pa) ,"
        "2 = Volts (v), "
        "3 = Millivolts (mV), "
        "4 = Amperes (A), "
        "5 = Meters (m), "
        "6 = Meters per second (m/s), "
        "7 = Meters per second squared (m/s2), "
        "8 = Newton (N), "
        "9 = Watt (W)"
    )

    device_trace_identifier = field(
        Int16, offset=213, default=0, documentation=
        "Device/Trace Identifier — The unit number or id number of the device associated with the Data Trace "
        "(i.e. 4368 for vibrator serial number 4368 or 20316 for gun 16 on string 3 on vessel 2). This field allows "
        "traces to be associated across trace ensembles independently of the trace number (ensemble_trace_num field)."
    )

    time_scalar = field(
        Int16, offset=215, default=0, documentation=
        "Scalar to be applied to times specified in Trace Header fields uphole_time_at_source, uphole_time_at_group,"
        "source_static_correction, group_static_correction, total_static, lag_time_a, lag_time_b, "
        "delay_recording_time, mute_start_time, mute_end_time to give the true time value in milliseconds. "
        "Scalar = 1, +10, +100, +1000, or +10,000. If positive, scalar is used as a multiplier; "
        "if negative, scalar is used as divisor. A value of zero is assumed to be a scalar value of 1."
    )

    source_type = field(
        Int16, offset=217, default=0, documentation=
        "Source Type/Orientation — Defines the type and the orientation of the energy source. The terms vertical, "
        "cross-line and in-line refer to the three axes of an orthogonal coordinate system. The absolute azimuthal "
        "orientation of the coordinate system axes can be defined in the Bin Grid Definition Stanza."
        "-1 to -n = Other (should be described in Source Type/Orientation stanza), "
        "0 = Unknown, "
        "1 = Vibratory - Vertical orientation, "
        "2 = Vibratory - Cross-line orientation, "
        "3 = Vibratory - In-line orientation, "
        "4 = Impulsive - Vertical orientation, "
        "5 = Impulsive - Cross-line orientation, "
        "6 = Impulsive - In-line orientation, "
        "7 = Distributed Impulsive - Vertical orientation, "
        "8 = Distributed Impulsive - Cross-line orientation, "
        "9 = Distributed Impulsive - In-line orientation, "
    )

    source_energy_direction = field(
        Int32, offset=219, default=0, documentation=  # TODO: This is six bytes. What is the format?
        "Source Energy Direction with respect to the source orientation. The positive orientation direction is "
        "defined in Bytes 217-218 of the Trace Header. The energy direction is encoded in tenths of degrees "
        "(i.e. 347.8° is encoded as 3478)."
    )

    source_measurement_mantissa = field(
        Int32, offset=225, default=0, documentation=
        "Source Measurement mantissa. Describes the source effort used to generate the trace. The measurement can be "
        "simple, qualitative measurements such as the total weight of explosive used or the peak air gun pressure or "
        "the number of vibrators times the sweep duration. Although these simple measurements are acceptable, it is "
        "preferable to use true measurement units of energy or work. The constant is encoded as a four-byte, two's "
        "complement integer (source_measurement_mantissa) and a two-byte, two's complement integer "
        "source_measurement_exponent) which is the power of ten exponent "
        "(i.e. source_measurement_mantissa * 10**source_measurement_exponent)."
    )

    source_measurement_exponent = field(
        Int16, offset=229, default=0, documentation=
        "Source Measurement exponent. Describes the source effort used to generate the trace. The measurement can be "
        "simple, qualitative measurements such as the total weight of explosive used or the peak air gun pressure or "
        "the number of vibrators times the sweep duration. Although these simple measurements are acceptable, it is "
        "preferable to use true measurement units of energy or work. The constant is encoded as a four-byte, two's "
        "complement integer (source_measurement_mantissa) and a two-byte, two's complement integer "
        "source_measurement_exponent) which is the power of ten exponent "
        "(i.e. source_measurement_mantissa * 10**source_measurement_exponent)."
    )

    source_measurement_unit = field(
        Int16, offset=231, default=0, documentation=
        "Source Measurement Unit. The unit used for the source measurement."
        "-1 = Other (should be described in Source Measurement Unit stanza), "
        "0 = Unknown, "
        "1 = Joule (J), "
        "2 = Kilowatt (kW), "
        "3 = Pascal (Pa), "
        "4 = Bar (Bar), "
        "4 = Bar-meter (Bar-m), "
        "5 = Newton (N), "
        "6 = Kilograms (kg)"
    )



#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# basic functions for processing python dictionary

import sys, os
import numpy as np
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.messager import messager as vis_msg


__all__ = ['matdict']


def isDictConstantRow(dict):
    """
    Check if the dictionary has the same number of rows in all keys

    Args:
        dict:   Matrix dictionary

    Returns:
        True or false
    """

    # Check if the input dictionary be empty
    if len(dict.keys()) <= 0:
        vis_msg.print('ERROR in isDictConstantRow: Empty dictionary', type='error')
        sys.exit()

    firstkey = list(dict.keys())[0]
    nrow = len(dict[firstkey])

    for key in dict.keys():
        if nrow != len(dict[key]):
            vis_msg.print('ERROR in isDictConstantRow: ' + key + ' of inconstant row number',
                          type='error')
            return False

    return True


def maxDictConstantRow(dict):
    """
    Return the maximum row number of a dictionary

    Args:
        dict:   A given dictionary

    Returns:
        Maximum row number
    """

    # Check if the input dictionary be empty
    if len(dict.keys()) <= 0:
        vis_msg.print('WARNING in maxMatDictConstantRow: Empty dictionary',
                      type='error')
        return 0

    firstkey = list(dict.keys())[0]
    nrow = len(dict[firstkey])

    for key in dict.keys():
        if nrow > len(dict[key]):
            nrow = len(dict[key])

    return nrow


def filterDictByValue(dict, key, value=0, flag='=='):
    """
    Filter a dictionary by specified key value

    Args:
        dict:   Input dictionary
        key:    Key for filtering
        value:  Filtering threshold. Default is 0
        flag:   Filter operator: '>=', '<=', '==', '>', '<', '!='. Default is '=='

    Returns:
        A dictionary of specified index
    """

    # Check if the input dictionary be empty
    if len(dict.keys()) <= 0:
        vis_msg.print('ERROR in filterDictByValue: Empty dictionary', type='error')
        sys.exit()
    if isDictConstantRow(dict) is False:
        vis_msg.print('ERROR in filterDictByValue: Inconstant row number found in input dictionary', type='error')
        sys.exit()

    # Check key
    if (key in dict.keys()) is False:
        vis_msg.print('ERROR in filterDictByValue: Specified key not found in input dictionary', type='error')
        sys.exit()

    idx = []
    if flag == '>=':
        idx = [k for k, v in enumerate(dict[key]) if v >= value]
    if flag == '<=':
        idx = [k for k, v in enumerate(dict[key]) if v <= value]
    if flag == '==':
        idx = [k for k, v in enumerate(dict[key]) if v == value]
    if flag == '>':
        idx = [k for k, v in enumerate(dict[key]) if v > value]
    if flag == '<':
        idx = [k for k, v in enumerate(dict[key]) if v < value]
    if flag == '!=':
        idx = [k for k, v in enumerate(dict[key]) if v != value]

    if len(idx) <= 0:
        vis_msg.print('WARNING in filterDictByValue: No data found', type='warning')
        return {}

    batch = retrieveDictByIndex(dict, idx)

    return batch


def retrieveDictByKey(dict, key_list):
    """
    Retrieve a given dictionary by specified keys

    Args:
        dict:       Input dictionary
        key_list:   List of keys for retrieval

    Returns:
        A dictionary with the specified keys
    """

    # Check if the input dictionary be empty
    if len(dict.keys()) <= 0:
        vis_msg.print('ERROR in retrieveDictByKeys: Empty dictionary', type='error')
        sys.exit()
    if len(key_list) <= 0:
        vis_msg.print('ERROR in retrieveDictByKeys: Empty key list for retrieval', type='error')
        sys.exit()

    batch = {}
    for key in key_list:
        if key in dict.keys():
            batch[key] = dict[key]
        else:
            vis_msg.print('WARNING in retrieveDictByKeys: ' + key + ' Not found in the input dictionary',
                          type='warning')

    return batch


def retrieveDictByIndex(dict, index_list):
    """
    Retrieve a dictionary by specified index

    Args:
        dict:       Input dictionary, first row of index 0
        index_list: 1D array of index.

    Returns:
        A dictionary of specified index
    """

    # Check if the input dictionary be empty
    if len(dict.keys()) <= 0:
        vis_msg.print('ERROR in retrieveDictByIndex: Empty dictionary', type='error')
        sys.exit()
    if isDictConstantRow(dict) is False:
        vis_msg.print('ERROR in retrieveDictByIndex: Inconstant row number found in input dictionary',
                      type='error')
        sys.exit()

    # Check if the index be empty
    if np.ndim(index_list) < 1:
        vis_msg.print('ERROR in retrieveDictByIndex: 1D index array expected', type='error')
        sys.exit()
    if len(index_list) <= 0:
        vis_msg.print('ERROR in retrieveDictByIndex: Empty index array', type='error')
        sys.exit()

    maxnrow = len(dict[list(dict.keys())[0]])
    if np.max(index_list) >= maxnrow:
        vis_msg.print('ERROR in retrieveDictByIndex: Index list out of bound', type='error')
        sys.exit()

    # Extract all keys
    batch = {}
    for key in dict.keys():
        data_batch = [dict[key][int(i)] for i in index_list]
        data_batch = np.asarray(data_batch)
        batch[key] = data_batch

    return batch


def retrieveDictRandom(dict, batch_size=10, unique=True):
    """
    Retrieve a batch of a given dictionary randomly

    Args:
        dict:       Input dictionary
        batch_size: Size of dictionary batch. Default is 10
        unique:     Unique or not. If so, length of returned dictionary may be shorter than batch_size

    Returns:
        Dictionary batch
    """

    # Check if the input dictionary be empty
    if len(dict.keys()) <= 0:
        vis_msg.print('ERROR in retrieveDictBatchRandom: Empty dictionary', type='error')
        sys.exit()
    if isDictConstantRow(dict) is False:
        vis_msg.print('ERROR in retrieveDictBatchRandom: Inconstant row number found in input dictionary',
                      type='error')
        sys.exit()

    maxnrow = len(dict[list(dict.keys())[0]])

    if batch_size > maxnrow:
        vis_msg.print('WARNING in retrieveDictBatchRandom: Batch size too large', type='warning')
        batch_size = maxnrow

    # Random shuffle
    # idx = np.arange(0, maxnrow)
    # np.random.shuffle(idx)
    # idx = idx[0:batch_size]
    idx = np.random.randint(maxnrow, size=batch_size)
    if unique is True:
        idx = np.unique(idx)

    # Extract all keys
    batch = {}
    for key in dict.keys():
        data_batch = [dict[key][i] for i in idx]
        data_batch = np.asarray(data_batch)
        batch[key] = data_batch

    return batch


def splitDictRandom(dict, fraction=0.9):
    """
    Split a dict randomly

    Args:
        dict:       Input dictionary
        fraction:   Approximate fraction of the rows between 0 and 1

    Returns:
        Two new dictionaries
    """

    # Check if the input dictionary be empty
    if len(dict.keys()) <= 0:
        vis_msg.print('ERROR in splitDictRandom: Empty dictionary', type='error')
        sys.exit()
    if isDictConstantRow(dict) is False:
        vis_msg.print('ERROR in splitDictRandom: Inconstant row number found in input dictionary', type='error')
        sys.exit()

    if fraction <= 0.0 or fraction >= 1.0:
        vis_msg.print('ERROR in splitDictRandom: Fraction be (0.0, 1.0)', type='error')
        sys.exit()

    maxnrow = len(dict[list(dict.keys())[0]])

    nsplit = int(maxnrow * fraction)
    if nsplit <= 0:
        nsplit = 1
    if nsplit >= maxnrow:
        nsplit = maxnrow-1

    # Random shuffle
    idx = np.arange(0, maxnrow)
    np.random.shuffle(idx)
    idx_1 = idx[0:nsplit]
    idx_2 = idx[nsplit: maxnrow]

    # Extract all keys
    batch_1 = {}
    batch_2 = {}
    for key in dict.keys():
        data_batch = [dict[key][i] for i in idx_1]
        batch_1[key] = np.asarray(data_batch)
        data_batch = [dict[key][i] for i in idx_2]
        batch_2[key] = np.asarray(data_batch)

    return batch_1, batch_2


def truncateDict(dict, length=10):
    """
    Truncate a dictionary to a given length, and maximum truncation for the given length <= 0

    Args:
        dict:   Input dictionary
        length: Truncation length. Default is 10
                Maximum truncation if the given length <= 0

    Returns:
        Truncated dictionary
    """

    # Check if the input dictionary be empty
    if len(dict.keys()) <= 0:
        vis_msg.print('ERROR in truncateDict: Empty dictionary', type='error')
        sys.exit()

    firstkey = list(dict.keys())[0]
    maxlen = np.shape(dict[firstkey])[0]
    for key in dict.keys():
        len_key = np.shape(dict[key])[0]
        if maxlen > len_key:
            maxlen = len_key

    if length<=0 or length>maxlen:
        vis_msg.print('WARNING in truncateDict: Maximum truncation performed', type='warning')
        length = maxlen

    idx = np.linspace(0, length-1, num=length, dtype=int)
    batch = {}
    for key in dict.keys():
        data_batch = [dict[key][i] for i in idx]
        data_batch = np.asarray(data_batch)
        batch[key] = data_batch

    return batch


def isMatDict(dict):
    """
    Check if all dictionary values are of type numpy.ndarray

    Args:
        dict:       The given dictionary

    Returns:
        True or false
    """

    # Check if the input dictionary be empty
    if len(dict.keys()) <= 0:
        vis_msg.print('ERROR in isMatDict: Empty input dictionary', type='error')
        sys.exit()

    for key in dict.keys():
        if type(dict[key]) != np.ndarray:
            return False

    return True


def extractMatDict(dict):
    """
    Extract the dictionary values of numpy.ndarray as a separate dictionary

    Args:
        dict:   a dictionary

    Returns:
        a dictionary with all values of type numpy.ndarray
    """

    if len(dict.keys()) <= 0:
        vis_msg.print('ERROR in extractNumDict: Empty input dictionary', type='error')
        sys.exit()

    numdict = {}
    for key in dict.keys():
        if type(dict[key]) == np.ndarray:
            numdict[key] = dict[key]

    return numdict


def mergeMatDict(dict1, dict2):
    """
    Merge two matrix dictionaries by their keys

    Args:
        dict1:  first matrix dictionary
        dict2: second matrix dictionary

    Returns:
        A new matrix dictionary
    """

    if (isMatDict(dict1) is False) or (isMatDict(dict2) is False):
        vis_msg.print('ERROR in mergeMatDict: Matrix dictionary expected', type='error')
        sys.exit()

    batch = dict1.copy()
    for key in dict2.keys():
        if (key in dict1.keys()):
            if np.shape(batch[key])[1] == np.shape(dict2[key])[1]:
                batch[key] = np.concatenate((batch[key], dict2[key]))
            else:
                vis_msg.print('WARNING: Inconstant matrix width in two dictionaries', type='warning')
        else:
            batch[key] = dict2[key]

    return batch


def exportMatDict(dict, key_list=None):
    """
    Export a matrix dictionary as a numpy matrix

    Args:
        dict:       input matrix dictionary
        key_list:   list of keys for export. Default is None to export all keys

    Returns:
        matrix
    """

    # Check if the input dictionary be empty
    if len(dict.keys()) <= 0:
        vis_msg.print('ERROR in exportMatDict: Empty dictionary', type='error')
        sys.exit()
    if isMatDict(dict) is False:
        vis_msg.print('ERROR in exportMatDict: Matrix dictionary expected', type='error')
        sys.exit()

    if key_list is None:
        vis_msg.print('WARNING in exportMatDict: Export all keys', type='warning')
        key_list = dict.keys()
    if len(key_list) <= 0:
        vis_msg.print('ERROR in exportMatDict: Empty key list for export', type='error')
        sys.exit()


    mat = []
    for key in key_list:
        if key in dict.keys():
            if len(mat) == 0:
                mat = dict[key]
            else:
                if np.shape(mat)[0] == np.shape(dict[key])[0]:
                    mat = np.concatenate((mat, dict[key]), axis=1)
                else:
                    vis_msg.print('ERROR in exportMatDict: ' + key + ' of inconstant row number', type='error')
                    sys.exit()

    return mat


def extendMatDict(dict, length=100):
    """
    Extend a matrix dictionary to a given length

    Args:
        dict:   input matrix dictionary
        length: extended length. Default is 100

    Returns:
        Extended dictionary
    """

    if isMatDict(dict) is False:
        vis_msg.print('ERROR in extendMatDict: Matrix dictionary expected', type='error')
        sys.exit()

    maxlen = maxDictConstantRow(dict)

    if length <= maxlen:
        vis_msg.print('WARNING in extendDict: No extension performed', type='warning')
        length = maxlen

    batch = dict.copy()
    while maxDictConstantRow(batch) < length:
        lendiff = length - maxDictConstantRow(batch)
        if lendiff > maxlen:
            batch0 = dict.copy()
        else:
            batch0 = retrieveDictRandom(dict, lendiff)
        batch = mergeMatDict(batch, batch0)

    return batch


class matdict:
    # Pack all functions as a class
    #
    isDictConstantRow = isDictConstantRow
    maxDictConstantRow = maxDictConstantRow
    #
    filterDictByValue = filterDictByValue
    #
    retrieveDictByIndex = retrieveDictByIndex
    retrieveDictByKey = retrieveDictByKey
    retrieveDictRandom = retrieveDictRandom
    #
    splitDictRandom = splitDictRandom
    truncateDict = truncateDict
    #
    isMatDict = isMatDict
    extractMatDict = extractMatDict
    mergeMatDict = mergeMatDict
    exportMatDict = exportMatDict
    extendMatDict = extendMatDict


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     February 2019                                                                   #
#                                                                                           #
#############################################################################################

# Create a window for import survey from a seismic segy


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.seismic.inputoutput import inputoutput as seis_io
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.gui.viewsegytextualheader import viewsegytextualheader as gui_viewsegytextualheader
from cognitivegeo.src.gui.viewsegybinaryheader import viewsegybinaryheader as gui_viewsegybinaryheader
from cognitivegeo.src.gui.viewsegytraceheader import viewsegytraceheader as gui_viewsegytraceheader
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class importsurveysegy(object):

    survinfo = {}
    rootpath = ''
    #
    iconpath = os.path.dirname(__file__)
    dialog = None
    #
    segyfile = ''

    def setupGUI(self, ImportSurveySegy):
        ImportSurveySegy.setObjectName("ImportSurveySegy")
        ImportSurveySegy.setFixedSize(480, 330)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/segy.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ImportSurveySegy.setWindowIcon(icon)
        self.lblsegy = QtWidgets.QLabel(ImportSurveySegy)
        self.lblsegy.setObjectName("lblsegy")
        self.lblsegy.setGeometry(QtCore.QRect(10, 10, 100, 30))
        self.ldtsegy = QtWidgets.QLineEdit(ImportSurveySegy)
        self.ldtsegy.setObjectName("ldtisegy")
        self.ldtsegy.setGeometry(QtCore.QRect(110, 10, 290, 30))
        self.btnsegy = QtWidgets.QPushButton(ImportSurveySegy)
        self.btnsegy.setObjectName("btnsegy")
        self.btnsegy.setGeometry(QtCore.QRect(410, 10, 60, 30))
        #
        self.lblview = QtWidgets.QLabel(ImportSurveySegy)
        self.lblview.setObjectName("lblview")
        self.lblview.setGeometry(QtCore.QRect(10, 50, 100, 30))
        self.btntextualheader = QtWidgets.QPushButton(ImportSurveySegy)
        self.btntextualheader.setObjectName("btntextualheader")
        self.btntextualheader.setGeometry(QtCore.QRect(30, 90, 140, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/view.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btntextualheader.setIcon(icon)
        self.btnbinaryheader = QtWidgets.QPushButton(ImportSurveySegy)
        self.btnbinaryheader.setObjectName("btnbinaryheader")
        self.btnbinaryheader.setGeometry(QtCore.QRect(180, 90, 140, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/view.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnbinaryheader.setIcon(icon)
        self.btntraceheader = QtWidgets.QPushButton(ImportSurveySegy)
        self.btntraceheader.setObjectName("btntraceheader")
        self.btntraceheader.setGeometry(QtCore.QRect(330, 90, 140, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/view.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btntraceheader.setIcon(icon)
        self.lblpara = QtWidgets.QLabel(ImportSurveySegy)
        self.lblpara.setObjectName("lbltraceheader")
        self.lblpara.setGeometry(QtCore.QRect(10, 140, 100, 30))
        self.lblx = QtWidgets.QLabel(ImportSurveySegy)
        self.lblx.setObjectName("lblx")
        self.lblx.setGeometry(QtCore.QRect(30, 180, 150, 30))
        self.cbbx = QtWidgets.QComboBox(ImportSurveySegy)
        self.cbbx.setObjectName("cbbx")
        self.cbbx.setGeometry(QtCore.QRect(180, 180, 60, 30))
        self.lbly = QtWidgets.QLabel(ImportSurveySegy)
        self.lbly.setObjectName("lbly")
        self.lbly.setGeometry(QtCore.QRect(260, 180, 150, 30))
        self.cbby = QtWidgets.QComboBox(ImportSurveySegy)
        self.cbby.setObjectName("cbby")
        self.cbby.setGeometry(QtCore.QRect(410, 180, 60, 30))
        self.lblinl = QtWidgets.QLabel(ImportSurveySegy)
        self.lblinl.setObjectName("lblinl")
        self.lblinl.setGeometry(QtCore.QRect(30, 220, 150, 30))
        self.cbbinl = QtWidgets.QComboBox(ImportSurveySegy)
        self.cbbinl.setObjectName("cbbinl")
        self.cbbinl.setGeometry(QtCore.QRect(180, 220, 60, 30))
        self.lblxl = QtWidgets.QLabel(ImportSurveySegy)
        self.lblxl.setObjectName("lblxl")
        self.lblxl.setGeometry(QtCore.QRect(260, 220, 150, 30))
        self.cbbxl = QtWidgets.QComboBox(ImportSurveySegy)
        self.cbbxl.setObjectName("cbbxl")
        self.cbbxl.setGeometry(QtCore.QRect(410, 220, 60, 30))
        #
        self.btnimport = QtWidgets.QPushButton(ImportSurveySegy)
        self.btnimport.setObjectName("btnimport")
        self.btnimport.setGeometry(QtCore.QRect(160, 270, 160, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/segy.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnimport.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ImportSurveySegy)
        self.msgbox.setObjectName("msgbox")
        _center_x = ImportSurveySegy.geometry().center().x()
        _center_y = ImportSurveySegy.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ImportSurveySegy)
        QtCore.QMetaObject.connectSlotsByName(ImportSurveySegy)


    def retranslateGUI(self, ImportSurveySegy):
        self.dialog = ImportSurveySegy
        #
        _translate = QtCore.QCoreApplication.translate
        ImportSurveySegy.setWindowTitle(_translate("ImportSurveySegy", "Import Survey from Seismic SEG-Y"))
        self.lblsegy.setText(_translate("ImportSurveySegy", "Select SEG-Y:"))
        self.ldtsegy.setText(_translate("ImportSurveySegy", os.path.abspath(self.rootpath)))
        self.btnsegy.setText(_translate("ImportSurveySegy", "Browse"))
        self.btnsegy.clicked.connect(self.clickBtnSegy)
        #
        self.lblview.setText(_translate("ImportSurveySegy", "View SEG-Y:"))
        self.btntextualheader.setText(_translate("ImportSurveySegy", "Textual Header"))
        self.btntextualheader.clicked.connect(self.clickBtnTextualHeader)
        self.btnbinaryheader.setText(_translate("ImportSurveySegy", "Binary Header"))
        self.btnbinaryheader.clicked.connect(self.clickBtnBinaryHeader)
        self.btntraceheader.setText(_translate("ImportSurveySegy", "Trace Header"))
        self.btntraceheader.clicked.connect(self.clickBtnTraceHeader)
        self.lblpara.setText(_translate("ImportSurveySegy", "Import settings:"))
        self.lblx.setText(_translate("ImportSurveySegy", "X coordinate at byte"))
        self.cbbx.addItems([str(i+1) for i in range(240)])
        self.cbbx.setCurrentIndex(180)
        self.lbly.setText(_translate("ImportSurveySegy", "Y coordinate at byte"))
        self.cbby.addItems([str(i + 1) for i in range(240)])
        self.cbby.setCurrentIndex(184)
        self.lblinl.setText(_translate("ImportSurveySegy", "Inline number at byte"))
        self.cbbinl.addItems([str(i + 1) for i in range(240)])
        self.cbbinl.setCurrentIndex(188)
        self.lblxl.setText(_translate("ImportSurveySegy", "Crossline number at byte"))
        self.cbbxl.addItems([str(i + 1) for i in range(240)])
        self.cbbxl.setCurrentIndex(192)
        #
        self.btnimport.setText(_translate("ImportSurveySegy", "Import SEG-Y"))
        self.btnimport.setDefault(True)
        self.btnimport.clicked.connect(self.clickBtnImport)


    def clickBtnSegy(self):
        _dialog = QtWidgets.QFileDialog()
        _file = _dialog.getOpenFileName(None, 'Select Seismic SEG-Y', self.rootpath,
                                         filter="SEG-Y files (*.segy; *.sgy);;All files (*.*)")
        if len(_file[0]) > 0:
            self.segyfile = _file[0]
            self.ldtsegy.setText(str(_file[0]))


    def clickBtnTextualHeader(self):
        self.refreshMsgBox()
        #
        if os.path.exists(self.segyfile) is False or os.path.isfile(self.segyfile) is False:
            vis_msg.print("ERROR in ImportSurveySegy: No SEG-Y selected for viewing textual header", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Import Survey from Seismic SEG-Y',
                                           'No SEG-Y selected for viewing textual header')
            return
        _viewheader = QtWidgets.QDialog()
        _gui = gui_viewsegytextualheader()
        _gui.segyfile = self.segyfile
        _gui.setupGUI(_viewheader)
        _viewheader.exec()
        _viewheader.show()


    def clickBtnBinaryHeader(self):
        self.refreshMsgBox()
        #
        if os.path.exists(self.segyfile) is False or os.path.isfile(self.segyfile) is False:
            vis_msg.print("ERROR in ImportSurveySegy: No SEG-Y selected for viewing binary header", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Import Survey from Seismic SEG-Y',
                                           'No SEG-Y selected for viewing binary header')
            return
        _viewheader = QtWidgets.QDialog()
        _gui = gui_viewsegybinaryheader()
        _gui.segyfile = self.segyfile
        _gui.setupGUI(_viewheader)
        _viewheader.exec()
        _viewheader.show()


    def clickBtnTraceHeader(self):
        self.refreshMsgBox()
        #
        if os.path.exists(self.segyfile) is False or os.path.isfile(self.segyfile) is False:
            vis_msg.print("ERROR in ImportSurveySegy: No SEG-Y selected for viewing trace header", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Import Survey from Seismic SEG-Y',
                                           'No SEG-Y selected for viewing trace header')
            return
        _viewheader = QtWidgets.QDialog()
        _gui = gui_viewsegytraceheader()
        _gui.segyfile = self.segyfile
        _gui.setupGUI(_viewheader)
        _viewheader.exec()
        _viewheader.show()


    def clickBtnImport(self):
        self.refreshMsgBox()
        #
        if os.path.exists(self.segyfile) is False or os.path.isfile(self.segyfile) is False:
            vis_msg.print("ERROR in ImportSurveySegy: No SEG-Y selected for import", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Import Survey from Seismic SEG-Y',
                                           'No SEG-Y selected for import')
            return
        # format trace header format
        _traceheaderformat = seis_io.defSegyTraceHeaderFormat(x_byte=self.cbbx.currentIndex()+1,
                                                              y_byte=self.cbby.currentIndex() + 1,
                                                              inl_byte=self.cbbinl.currentIndex()+1,
                                                              xl_byte=self.cbbxl.currentIndex()+1)
        #
        _survinfo = seis_io.readSeisInfoFromSegy(self.segyfile,
                                                 traceheaderformat=_traceheaderformat)
        print("SEG-Y survey information:")
        print("----- Dimension (inline x crossline x time/depth) : %d x %d x %d"
              % (_survinfo['ILNum'], _survinfo['XLNum'], _survinfo['ZNum']))
        print("----- Inline range (start, end, step): %d, %d, %d"
              % (_survinfo['ILStart'], _survinfo['ILEnd'], _survinfo['ILStep']))
        print("----- Crossline range (start, end, step): %d, %d, %d"
              % (_survinfo['XLStart'], _survinfo['XLEnd'], _survinfo['XLStep']))
        print("----- Time/depth range (start, end, step): %d, %d, %d"
              % (_survinfo['ZStart'], _survinfo['ZEnd'], _survinfo['ZStep']))
        #
        if seis_ays.checkSeisInfo(_survinfo):
            self.survinfo = _survinfo
        self.checkSurvInfo()
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Import Survey from Seismic SEG-Y",
                                          " Survey imported successfully")
        #
        return


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if checkSurvInfo(self.survinfo) is False:
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Import Seismic SEG-Y',
                                           'No survey found')
            return


def checkSurvInfo(survinfo):
    return seis_ays.checkSeisInfo(survinfo)


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    ImportSurveySegy = QtWidgets.QWidget()
    gui = importsurveysegy()
    gui.setupGUI(ImportSurveySegy)
    ImportSurveySegy.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# Create a window for import pre-stack seismic segys


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.psseismic.inputoutput import inputoutput as psseis_io
from cognitivegeo.src.psseismic.analysis import analysis as psseis_ays
from cognitivegeo.src.gui.viewsegytextualheader import viewsegytextualheader as gui_viewsegytextualheader
from cognitivegeo.src.gui.viewsegybinaryheader import viewsegybinaryheader as gui_viewsegybinaryheader
from cognitivegeo.src.gui.viewsegytraceheader import viewsegytraceheader as gui_viewsegytraceheader
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class importpsseissegy(object):

    psseisdata = {}
    rootpath = ''
    #
    iconpath = os.path.dirname(__file__)
    dialog = None
    #
    segylist = []

    def setupGUI(self, ImportPsSeisSegy):
        ImportPsSeisSegy.setObjectName("ImportPsSeisSegy")
        ImportPsSeisSegy.setFixedSize(480, 280)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/segy.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ImportPsSeisSegy.setWindowIcon(icon)
        self.lblsegy = QtWidgets.QLabel(ImportPsSeisSegy)
        self.lblsegy.setObjectName("lblsegy")
        self.lblsegy.setGeometry(QtCore.QRect(10, 10, 100, 30))
        self.ldtsegy = QtWidgets.QLineEdit(ImportPsSeisSegy)
        self.ldtsegy.setObjectName("ldtisegy")
        self.ldtsegy.setGeometry(QtCore.QRect(110, 10, 290, 30))
        self.btnsegy = QtWidgets.QPushButton(ImportPsSeisSegy)
        self.btnsegy.setObjectName("btnsegy")
        self.btnsegy.setGeometry(QtCore.QRect(410, 10, 60, 30))
        #
        self.lblview = QtWidgets.QLabel(ImportPsSeisSegy)
        self.lblview.setObjectName("lblview")
        self.lblview.setGeometry(QtCore.QRect(10, 50, 100, 30))
        self.btntextualheader = QtWidgets.QPushButton(ImportPsSeisSegy)
        self.btntextualheader.setObjectName("btntextualheader")
        self.btntextualheader.setGeometry(QtCore.QRect(30, 90, 140, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/view.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btntextualheader.setIcon(icon)
        self.btnbinaryheader = QtWidgets.QPushButton(ImportPsSeisSegy)
        self.btnbinaryheader.setObjectName("btnbinaryheader")
        self.btnbinaryheader.setGeometry(QtCore.QRect(180, 90, 140, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/view.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnbinaryheader.setIcon(icon)
        self.btntraceheader = QtWidgets.QPushButton(ImportPsSeisSegy)
        self.btntraceheader.setObjectName("btntraceheader")
        self.btntraceheader.setGeometry(QtCore.QRect(330, 90, 140, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/view.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btntraceheader.setIcon(icon)
        #
        self.lblpara = QtWidgets.QLabel(ImportPsSeisSegy)
        self.lblpara.setObjectName("lbltraceheader")
        self.lblpara.setGeometry(QtCore.QRect(10, 140, 100, 30))
        self.lblrecord = QtWidgets.QLabel(ImportPsSeisSegy)
        self.lblrecord.setObjectName("lblrecord")
        self.lblrecord.setGeometry(QtCore.QRect(30, 180, 150, 30))
        self.cbbrecord = QtWidgets.QComboBox(ImportPsSeisSegy)
        self.cbbrecord.setObjectName("cbbrecord")
        self.cbbrecord.setGeometry(QtCore.QRect(180, 180, 60, 30))
        self.lbltrace = QtWidgets.QLabel(ImportPsSeisSegy)
        self.lbltrace.setObjectName("lbltrace")
        self.lbltrace.setGeometry(QtCore.QRect(260, 180, 150, 30))
        self.cbbtrace = QtWidgets.QComboBox(ImportPsSeisSegy)
        self.cbbtrace.setObjectName("cbbtrace")
        self.cbbtrace.setGeometry(QtCore.QRect(410, 180, 60, 30))
        #
        self.btnimport = QtWidgets.QPushButton(ImportPsSeisSegy)
        self.btnimport.setObjectName("btnimport")
        self.btnimport.setGeometry(QtCore.QRect(160, 230, 160, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/segy.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnimport.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ImportPsSeisSegy)
        self.msgbox.setObjectName("msgbox")
        _center_x = ImportPsSeisSegy.geometry().center().x()
        _center_y = ImportPsSeisSegy.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ImportPsSeisSegy)
        QtCore.QMetaObject.connectSlotsByName(ImportPsSeisSegy)


    def retranslateGUI(self, ImportPsSeisSegy):
        self.dialog = ImportPsSeisSegy
        #
        _translate = QtCore.QCoreApplication.translate
        ImportPsSeisSegy.setWindowTitle(_translate("ImportPsSeisSegy", "Import Pre-stack Seismic SEG-Y"))
        self.lblsegy.setText(_translate("ImportPsSeisSegy", "Select SEG-Y(s):"))
        self.ldtsegy.setText(_translate("ImportPsSeisSegy", os.path.abspath(self.rootpath)))
        self.btnsegy.setText(_translate("ImportPsSeisSegy", "Browse"))
        self.btnsegy.clicked.connect(self.clickBtnSegy)
        #
        self.lblview.setText(_translate("ImportPsSeisSegy", "View 1st SEG-Y:"))
        self.btntextualheader.setText(_translate("ImportPsSeisSegy", "Textual Header"))
        self.btntextualheader.clicked.connect(self.clickBtnTextualHeader)
        self.btnbinaryheader.setText(_translate("ImportPsSeisSegy", "Binary Header"))
        self.btnbinaryheader.clicked.connect(self.clickBtnBinaryHeader)
        self.btntraceheader.setText(_translate("ImportPsSeisSegy", "Trace Header"))
        self.btntraceheader.clicked.connect(self.clickBtnTraceHeader)
        #
        self.lblpara.setText(_translate("ImportPsSeisSegy", "Import settings:"))
        self.lblrecord.setText(_translate("ImportPsSeisSegy", "Record number at byte"))
        self.cbbrecord.addItems([str(i + 1) for i in range(240)])
        self.cbbrecord.setCurrentIndex(8)
        self.lbltrace.setText(_translate("ImportPsSeisSegy", "Trace number at byte"))
        self.cbbtrace.addItems([str(i + 1) for i in range(240)])
        self.cbbtrace.setCurrentIndex(12)
        #
        self.btnimport.setText(_translate("ImportPsSeisSegy", "Import SEG-Y"))
        self.btnimport.clicked.connect(self.clickBtnImport)


    def clickBtnSegy(self):
        _dialog = QtWidgets.QFileDialog()
        _file = _dialog.getOpenFileNames(None, 'Select Pre-stack Seismic SEG-Y(s)', self.rootpath,
                                         filter="SEG-Y files (*.segy; *.sgy);;All files (*.*)")
        if len(_file[0]) > 0:
            self.segylist = _file[0]
            self.ldtsegy.setText(str(_file[0]))


    def clickBtnTextualHeader(self):
        self.refreshMsgBox()
        #
        _nfile = len(self.segylist)
        if _nfile <= 0:
            vis_msg.print("ERROR in ImportPsSeisSegy: No SEG-Y selected for viewing textual header",
                          type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Import Pre-stack Seismic SEG-Y',
                                           'No SEG-Y selected for viewing textual header')
            return
        _viewheader = QtWidgets.QDialog()
        _gui = gui_viewsegytextualheader()
        _gui.segyfile = self.segylist[0]
        _gui.setupGUI(_viewheader)
        _viewheader.exec()
        _viewheader.show()


    def clickBtnBinaryHeader(self):
        self.refreshMsgBox()
        #
        _nfile = len(self.segylist)
        if _nfile <= 0:
            vis_msg.print("ERROR in ImportPsSeisSegy: No SEG-Y selected for viewing binary header",
                          type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Import Pre-stack Seismic SEG-Y',
                                           'No SEG-Y selected for viewing binary header')
            return
        _viewheader = QtWidgets.QDialog()
        _gui = gui_viewsegybinaryheader()
        _gui.segyfile = self.segylist[0]
        _gui.setupGUI(_viewheader)
        _viewheader.exec()
        _viewheader.show()


    def clickBtnTraceHeader(self):
        self.refreshMsgBox()
        #
        _nfile = len(self.segylist)
        if _nfile <= 0:
            vis_msg.print("ERROR in ImportPsSeisSegy: No SEG-Y selected for viewing trace header",
                          type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Import SEG-Y',
                                           'No SEG-Y selected for viewing trace header')
            return
        _viewheader = QtWidgets.QDialog()
        _gui = gui_viewsegytraceheader()
        _gui.segyfile = self.segylist[0]
        _gui.setupGUI(_viewheader)
        _viewheader.exec()
        _viewheader.show()


    def clickBtnImport(self):
        self.refreshMsgBox()
        #
        _nfile = len(self.segylist)
        if _nfile <= 0:
            vis_msg.print("ERROR in ImportPsSeisSegy: No SEG-Y selected for import", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Import Pre-stack Seismic SEG-Y',
                                           'No SEG-Y selected for import')
            return
        # format trace header format
        _traceheaderformat = psseis_io.defSegyTraceHeaderFormat(record_byte=self.cbbrecord.currentIndex() + 1,
                                                                trace_byte=self.cbbtrace.currentIndex() + 1)
        #
        # Progress dialog
        _pgsdlg = QtWidgets.QProgressDialog()
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/segy.png")),
                       QtGui.QIcon.Normal, QtGui.QIcon.Off)
        _pgsdlg.setWindowIcon(icon)
        _pgsdlg.setWindowTitle('Import ' + str(_nfile) + ' Pre-stack Seismic SEG-Y')
        _pgsdlg.setCancelButton(None)
        _pgsdlg.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
        _pgsdlg.forceShow()
        _pgsdlg.setFixedWidth(400)
        #
        _psseisdata = {}
        #
        for i in range(_nfile):
            #
            _pgsdlg.setWindowTitle('Import ' + str(i + 1) + ' of ' + str(_nfile) + ' Pre-stack Seismic SEG-Y')
            #
            _filename = self.segylist[i]
            print("ImportPsSeisSegy: Import %d of %d SEG-Y: %s" % (i + 1, _nfile, _filename))
            _filenamemain = os.path.splitext(os.path.basename(_filename))[0]
            _psseisdata[_filenamemain] = psseis_io.readPsSeisFromSegy(_filename,
                                                                      traceheaderformat=_traceheaderformat,
                                                                      qpgsdlg=_pgsdlg)
            #
        # add new data to psseisdata
        for key in _psseisdata.keys():
            if key in self.psseisdata.keys()and checkPsSeisData(self.psseisdata[key]):
                reply = QtWidgets.QMessageBox.question(self.msgbox, 'Import Pre-stack Seismic SEG-Y',
                                                       key + ' already exists. Overwrite?',
                                                       QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                       QtWidgets.QMessageBox.No)
                if reply == QtWidgets.QMessageBox.No:
                    return
            self.psseisdata[key] = _psseisdata[key]
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Import Pre-stack Seismic SEG-Y",
                                          str(_nfile) + " SEG-Y imported successfully")
        #
        return


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


def checkPsSeisData(psseisdata):
    return psseis_ays.checkPsSeis(psseisdata)


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    ImportPsSeisSegy = QtWidgets.QWidget()
    gui = importpsseissegy()
    gui.setupGUI(ImportPsSeisSegy)
    ImportPsSeisSegy.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# Create a GUI for player configuration

from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
import numpy as np
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.player import player as vis_player


QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class configplayer(object):

    playerconfig = {}
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ConfigPlayer):
        ConfigPlayer.setObjectName("ConfigPlayer")
        ConfigPlayer.setFixedSize(820, 280)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/video.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ConfigPlayer.setWindowIcon(icon)
        #
        self.twgplayer = QtWidgets.QTableWidget(ConfigPlayer)
        self.twgplayer.setObjectName("twgplayer")
        self.twgplayer.setGeometry(QtCore.QRect(10, 10, 800, 200))
        self.twgplayer.setColumnCount(9)
        self.twgplayer.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.twgplayer.verticalHeader().hide()
        #
        self.btnapply = QtWidgets.QPushButton(ConfigPlayer)
        self.btnapply.setObjectName("btnapply")
        self.btnapply.setGeometry(QtCore.QRect(360, 230, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ConfigPlayer)
        self.msgbox.setObjectName("msgbox")
        _center_x = ConfigPlayer.geometry().center().x()
        _center_y = ConfigPlayer.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ConfigPlayer)
        QtCore.QMetaObject.connectSlotsByName(ConfigPlayer)


    def retranslateGUI(self, ConfigPlayer):
        self.dialog = ConfigPlayer
        #
        _translate = QtCore.QCoreApplication.translate
        ConfigPlayer.setWindowTitle(_translate("ConfigPlayer", "Player Configuration"))
        #
        self.twgplayer.setHorizontalHeaderLabels(["", '|<--', "<<--", '<<<<', '>>||',
                                                  '>>>>', '-->>', '-->|', 'Interval'])
        self.twgplayer.setRowCount(len(self.playerconfig.keys()))
        _idx = 0
        for play in sorted(self.playerconfig.keys()):
            item = QtWidgets.QTableWidgetItem()
            item.setText(play)
            item.setTextAlignment(QtCore.Qt.AlignCenter)
            self.twgplayer.setItem(_idx, 0, item)
            #
            for prop in vis_player.PlayerPropertyList[:-1]:
                _idxprop = list.index(vis_player.PlayerPropertyList, prop)
                item = QtWidgets.QComboBox()
                item.addItems(vis_player.PlayerKeyList)
                item.setCurrentIndex(list.index(vis_player.PlayerKeyList,
                                                self.playerconfig[play][prop]))
                self.twgplayer.setCellWidget(_idx, _idxprop+1, item)
            prop = vis_player.PlayerPropertyList[-1]
            _idxprop = list.index(vis_player.PlayerPropertyList, prop)
            item = QtWidgets.QComboBox()
            item.addItems([str(t) for t in vis_player.PlayerIntervalList])
            item.setCurrentIndex(list.index(vis_player.PlayerIntervalList,
                                            self.playerconfig[play][prop]))
            self.twgplayer.setCellWidget(_idx, _idxprop + 1, item)
            #
            _idx = _idx + 1
        #
        self.btnapply.setText(_translate("ConfigPlayer", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnApply)


    def clickBtnApply(self):
        self.refreshMsgBox()
        #
        for player in sorted(self.playerconfig.keys()):
            _idx = list.index(sorted(self.playerconfig.keys()), player)
            _config = {}
            _config['First'] = vis_player.PlayerKeyList[self.twgplayer.cellWidget(_idx, 1).currentIndex()]
            _config['Previous'] = vis_player.PlayerKeyList[self.twgplayer.cellWidget(_idx, 2).currentIndex()]
            _config['Backward'] = vis_player.PlayerKeyList[self.twgplayer.cellWidget(_idx, 3).currentIndex()]
            _config['Pause'] = vis_player.PlayerKeyList[self.twgplayer.cellWidget(_idx, 4).currentIndex()]
            _config['Forward'] = vis_player.PlayerKeyList[self.twgplayer.cellWidget(_idx, 5).currentIndex()]
            _config['Next'] = vis_player.PlayerKeyList[self.twgplayer.cellWidget(_idx, 6).currentIndex()]
            _config['Last'] = vis_player.PlayerKeyList[self.twgplayer.cellWidget(_idx, 7).currentIndex()]
            _config['Interval'] = vis_player.PlayerIntervalList[self.twgplayer.cellWidget(_idx, 8).currentIndex()]
            self.playerconfig[player] = _config
        #
        self.dialog.close()


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ConfigPlayer = QtWidgets.QWidget()
    gui = configplayer()
    gui.setupGUI(ConfigPlayer)
    ConfigPlayer.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# basic functions for player

import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.core.keyboard import keyboard as core_key


__all__ = ['player']


# List of all available player options, including Property, Key, and Interval
PlayerPropertyList = ['First', 'Previous', 'Backward', 'Pause', 'Forward', 'Next', 'Last', 'Interval']
PlayerKeyList = core_key.LetterKeyList
PlayerIntervalList = [1, 2, 5, 10, 20, 50, 100, 200, 500, 1000]


class player:
    # Pack all functions as a class
    #
    PlayerPropertyList = PlayerPropertyList
    PlayerKeyList = PlayerKeyList
    PlayerIntervalList = PlayerIntervalList


from collections import OrderedDict
from struct import Struct
from itertools import zip_longest
import os, sys
sys.path.append(os.path.dirname(__file__)[:-6][:-4][:-13])
from cognitivegeo.src.segpy.datatypes import SEG_Y_TYPE_TO_CTYPE
from cognitivegeo.src.segpy.util import pairwise, intervals_partially_overlap, complementary_intervals


def size_of(t):
    return t.SIZE


def compile_struct(header_format_class, start_offset=0, length_in_bytes=None, endian='>'):
    """Compile a struct description from a record.

    Args:
        header_format_class: A header_format class.

        start_offset: Optional start offset for the header in bytes.  Indicates the position of the start of
            the header in the same reference frame as which the field offsets are given.

        length_in_bytes: Optional length in bytes for the header. If the supplied header described a format shorter
            than this value the returned format will be padded with placeholders for bytes to be discarded. If the
            value is less than the minimum required for the format described by header_format_class an error will be
            raised.

        endian: '>' for big-endian data (the standard and default), '<'
            for little-endian (non-standard).

    Returns:
        A two-tuple containing in the zeroth element a format string which can be used with the struct.unpack function,
        and in the second element containing a list-of-lists for field names.  Each item in the outer list corresponds
        to an element of the tuple of data values returned by struct.unpack(); each name associated with that index is a
        field to which the unpacked value should be assigned.

        format, allocations = compile_struct(TraceHeaderFormat)
        values = struct.unpack(format)
        field_names_to_values = {}
        for field_names, value in zip(allocations, values):
            for field_name in field_names:
                field_names_to_values[field_name] = value
        header = Header(**field_names_to_values)

    Raises:
        ValueError: If header_format_class defines no fields.
        ValueError: If header_format_class contains fields which overlap but are not exactly coincident.
        ValueError: If header_format_class contains coincident fields of different types.
        ValueError: If header_format_class described a format longer than length_in_bytes.

    """
    fields = [getattr(header_format_class, name) for name in header_format_class.ordered_field_names()]

    sorted_fields = sorted(fields, key=lambda f: f.offset)

    if len(sorted_fields) < 1:
        raise ValueError("Header format class {!r} defines no fields".format(header_format_class.__name__))

    if len(sorted_fields) > 1:
        for a, b in pairwise(sorted_fields):
            if intervals_partially_overlap(range(a.offset, a.offset + size_of(a.value_type)),
                                           range(b.offset, b.offset + size_of(b.value_type))):
                raise ValueError("Record fields {!r} at offset {} and {!r} at offset {} are distinct but overlap."
                                  .format(a.name, a.offset, b.name, b.offset))

    last_field = sorted_fields[-1]
    defined_length = (last_field.offset - start_offset) + size_of(last_field.value_type)
    specified_length = defined_length if (length_in_bytes is None) else length_in_bytes
    padding_length = specified_length - defined_length
    if padding_length < 0:
        raise ValueError("Header length {!r} bytes defined by {!r} is less than specified length in bytes {!r}"
                         .format(defined_length, header_format_class.__name__, specified_length))

    offset_to_fields = OrderedDict()
    for field in sorted_fields:
        relative_offset = field.offset - start_offset  # relative_offser is zero-based
        if relative_offset not in offset_to_fields:
            offset_to_fields[relative_offset] = []
        if len(offset_to_fields[relative_offset]) > 0:
            if offset_to_fields[relative_offset][0].value_type is not field.value_type:
                # TODO: Test this error handling
                raise ValueError("Coincident fields {!r} and {!r} at offset {} have different types {!r} and {!r}"
                                  .format(offset_to_fields[relative_offset][0],
                                          field,
                                          offset_to_fields[relative_offset][0].offset,
                                          offset_to_fields[relative_offset][0].value_type,
                                          field.value_type))
        offset_to_fields[relative_offset].append(field)

    # Create a list of ranges where each range spans the byte indexes covered by each field
    field_spans = [range(offset, offset + size_of(fields[0].value_type))
                   for offset, fields in offset_to_fields.items()]

    gap_intervals = complementary_intervals(field_spans, start=0, stop=specified_length)  # One-based indexes

    # Create a format string usable with the struct module
    format_chunks = [endian]
    representative_fields = (fields[0] for fields in offset_to_fields.values())
    for gap_interval, field in zip_longest(gap_intervals, representative_fields, fillvalue=None):
        gap_length = len(gap_interval)
        if gap_length > 0:
            format_chunks.append('x' * gap_length)
        if field is not None:
            format_chunks.append(SEG_Y_TYPE_TO_CTYPE[field.value_type.SEG_Y_TYPE])
    cformat = ''.join(format_chunks)

    # Create a list of mapping item index to field names.
    # [0] -> ['field_1', 'field_2']
    # [1] -> ['field_3']
    # [2] -> ['field_4']
    field_name_allocations = [[field.name for field in fields]
                              for fields in offset_to_fields.values()]
    return cformat, field_name_allocations


class HeaderPacker:
    """Packing and unpacking header instances."""

    def __init__(self, header_format_class, endian='>'):
        self._header_format_class = header_format_class
        self._format, self._field_name_allocations = compile_struct(
            header_format_class,
            header_format_class.START_OFFSET_IN_BYTES,
            header_format_class.LENGTH_IN_BYTES,
            endian)
        self._struct = Struct(self._format)

    @property
    def header_format_class(self):
        return self._header_format_class

    def pack(self, header):
        """Pack a header into a buffer.
        """
        if not isinstance(header, self._header_format_class):
            raise TypeError("{}({}) cannot pack header of type {}.".format(
                self.__class__.__name__,
                self._header_format_class.__name__,
                header.__class__.__name__
            ))
        values = [getattr(header, names[0]) for names in self._field_name_allocations]
        return self._struct.pack(*values)

    def unpack(self, buffer):
        """Unpack a header into a header object.

        Overwrites any existing header field values with new values
        obtained from the buffer.

        Returns:
            The header object.
        """
        values = self._struct.unpack(buffer)

        kwargs = {name: value
                  for names, value in zip(self._field_name_allocations, values)
                  for name in names}

        return self._header_format_class(**kwargs)

    def __repr__(self):
        return "{}({})".format(
            self.__class__.__name__,
            self._header_format_class.__name__)


def main():
    from segpy.trace_header import TraceHeaderRev0
    compile_struct(TraceHeaderRev0, 1, 240)
    pass

if __name__ == '__main__':
    main()



#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# basic functions for processing data

import numpy as np
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.messager import messager as vis_msg


__all__ = ['data']


def str2int(str):
    """
    Covert a string to integer

    Args:
        str:    A string for conversion

    Return:
        The corresponding integer if it is convertable. Otherwise, False is returned.
    """

    try:
        return int(str)
    except ValueError:
        return False


def str2float(str):
    """
    Covert a string to float

    Args:
        str:    A string for conversion

    Return:
        The corresponding float if it is convertable. Otherwise, False is returned.
    """

    try:
        return float(str)
    except ValueError:
        return False


class data:
    # Pack all functions as a class
    str2int = str2int
    str2float = str2float



#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for retrieving seismic properties from a given numpy dictionary


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.pointset.analysis import analysis as point_ays
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.basic.matdict import matdict as basic_mdt
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class rtrvseisprop(object):

    pointsetdata = {}
    rootpath = ''
    #
    iconpath = os.path.dirname(__file__)
    dialog = None
    #
    npydata = None
    npyinfo = {}


    def setupGUI(self, RtrvSeisProp):
        RtrvSeisProp.setObjectName("RtrvSeisProp")
        RtrvSeisProp.setFixedSize(400, 390)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/retrieve.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        RtrvSeisProp.setWindowIcon(icon)
        #
        self.lblfrom = QtWidgets.QLabel(RtrvSeisProp)
        self.lblfrom.setObjectName("lblfrom")
        self.lblfrom.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.ldtfrom = QtWidgets.QLineEdit(RtrvSeisProp)
        self.ldtfrom.setObjectName("ldtfrom")
        self.ldtfrom.setGeometry(QtCore.QRect(160, 10, 160, 30))
        self.btnfrom = QtWidgets.QPushButton(RtrvSeisProp)
        self.btnfrom.setObjectName("btnfrom")
        self.btnfrom.setGeometry(QtCore.QRect(330, 10, 60, 30))
        self.lblattrib = QtWidgets.QLabel(RtrvSeisProp)
        self.lblattrib.setObjectName("lblattrib")
        self.lblattrib.setGeometry(QtCore.QRect(10, 50, 150, 30))
        self.lwgattrib = QtWidgets.QListWidget(RtrvSeisProp)
        self.lwgattrib.setObjectName("lwgattrib")
        self.lwgattrib.setGeometry(QtCore.QRect(160, 50, 230, 200))
        self.lwgattrib.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.lbldims = QtWidgets.QLabel(RtrvSeisProp)
        self.lbldims.setObjectName("lbldims")
        self.lbldims.setGeometry(QtCore.QRect(10, 270, 150, 30))
        self.ldtdimsinl = QtWidgets.QLineEdit(RtrvSeisProp)
        self.ldtdimsinl.setObjectName("ldtdimsinl")
        self.ldtdimsinl.setGeometry(QtCore.QRect(160, 270, 60, 30))
        self.ldtdimsxl = QtWidgets.QLineEdit(RtrvSeisProp)
        self.ldtdimsxl.setObjectName("ldtdimsxl")
        self.ldtdimsxl.setGeometry(QtCore.QRect(245, 270, 60, 30))
        self.ldtdimsz = QtWidgets.QLineEdit(RtrvSeisProp)
        self.ldtdimsz.setObjectName("ldtdimsz")
        self.ldtdimsz.setGeometry(QtCore.QRect(330, 270, 60, 30))
        self.btnrtrvprop = QtWidgets.QPushButton(RtrvSeisProp)
        self.btnrtrvprop.setObjectName("btnrtrvprop")
        self.btnrtrvprop.setGeometry(QtCore.QRect(120, 330, 160, 30))
        self.btnrtrvprop.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(RtrvSeisProp)
        self.msgbox.setObjectName("msgbox")
        _center_x = RtrvSeisProp.geometry().center().x()
        _center_y = RtrvSeisProp.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(RtrvSeisProp)
        QtCore.QMetaObject.connectSlotsByName(RtrvSeisProp)


    def retranslateGUI(self, RtrvSeisProp):
        self.dialog = RtrvSeisProp
        #
        _translate = QtCore.QCoreApplication.translate
        RtrvSeisProp.setWindowTitle(_translate("RtrvSeisProp", "Retrieve Seismic Property"))
        self.lblfrom.setText(_translate("RtrvSeisProp", "Select Seismic NumPy:"))
        self.ldtfrom.setText(_translate("RtrvSeisProp", ""))
        self.ldtfrom.textChanged.connect(self.changeLdtFrom)
        self.btnfrom.setText(_translate("RtrvSeisProp", "Browse"))
        self.btnfrom.clicked.connect(self.clickBtnFrom)
        self.lblattrib.setText(_translate("RtrvSeisProp", "Select target properties:"))
        self.lbldims.setText(_translate("RtrvSeisProp", "Retrieval radius (IL/XL/Z):"))
        self.ldtdimsinl.setText(_translate("RtrvSeisProp", "0"))
        self.ldtdimsinl.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtdimsxl.setText(_translate("RtrvSeisProp", "0"))
        self.ldtdimsxl.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtdimsz.setText(_translate("RtrvSeisProp", "0"))
        self.ldtdimsz.setAlignment(QtCore.Qt.AlignCenter)
        self.btnrtrvprop.setText(_translate("RtrvSeisProp", "Retrieve Property"))
        self.btnrtrvprop.clicked.connect(self.clickBtnRtrvSeisProp)


    def changeLdtFrom(self):
        self.refreshMsgBox()
        #
        self.lwgattrib.clear()
        #
        if os.path.exists(self.ldtfrom.text()) is False:
            vis_msg.print("ERROR in RtrvSeisProp: No NumPy selected for retrieval", type='error')
            return
        #
        try:
            self.npydata = np.load(self.ldtfrom.text()).item()
            if ('SeisInfo' not in self.npydata.keys())\
                    or (seis_ays.checkSeisInfo(self.npydata['SeisInfo']) is False):
                vis_msg.print("ERROR in RtrvSeisProp: NumPy dictionary contains no information about seismic survey",
                              type='error')
                QtWidgets.QMessageBox.critical(self.msgbox,
                                               'Retrieve Seismic Property',
                                               'NumPy dictionary contains no inforamtion about seismic')
                return
            self.npyinfo = self.npydata['SeisInfo']
            self.npydata.pop('SeisInfo')
            for key in self.npydata.keys():
                if checkSeisData(self.npydata[key], self.npyinfo) is False:
                    self.npydata.pop(key)
        except ValueError:
            _npydata = np.load(self.ldtfrom.text())
            _filename = os.path.splitext(os.path.basename(self.ldtfrom.text()))[0]
            if np.ndim(_npydata)<=1 or np.ndim(_npydata)>=4:
                vis_msg.print("ERROR in RtrvSeisProp: NumPy matrix shall be 2D or 3D", type='error')
                QtWidgets.QMessageBox.critical(self.msgbox,
                                               'Retrieve Seismic Property',
                                               'NumPy matrix shall be 2D or 3D')
                return
            if np.ndim(_npydata) == 2:
                if np.shape(_npydata)[1] < 4:
                    vis_msg.print("ERROR in RtrvSeisProp: 2D NumPy matrix shall contain at least 4 columns",
                                  type='error')
                    QtWidgets.QMessageBox.critical(self.msgbox,
                                                   'Retrieve Seismic Property',
                                                   '2D NumPy matrix shall contain at least 4 columns')
                    return
                self.npyinfo = seis_ays.getSeisInfoFrom2DMat(_npydata)
                _npydata = np.transpose(np.reshape(_npydata[:, 3:4], [self.npyinfo['ILNum'],
                                                                      self.npyinfo['XLNum'],
                                                                      self.npyinfo['ZNum']]), [2, 1, 0])
            if np.ndim(_npydata) == 3:
                self.npyinfo = seis_ays.createSeisInfoFrom3DMat(_npydata)
                _npydata = _npydata

            self.npydata = {}
            self.npydata[_filename] = _npydata
        #
        if seis_ays.checkSeisInfo(self.npyinfo) is False:
            vis_msg.print("ERROR in RtrvSeisProp: Selected not seismic numpy", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Retrieve Seismic Property',
                                           'Selected not seismic numpy')
            return
        #
        _firstattrib = None
        for i in sorted(self.npydata.keys()):
            item = QtWidgets.QListWidgetItem(self.lwgattrib)
            item.setText(i)
            self.lwgattrib.addItem(item)
            if _firstattrib is None:
                _firstattrib = item
        self.lwgattrib.setCurrentItem(_firstattrib)


    def clickBtnFrom(self):
        self.refreshMsgBox()
        #
        _dialog = QtWidgets.QFileDialog()
        _file = _dialog.getOpenFileName(None, 'Select Seismic NumPy', self.rootpath,
                                        filter="NumPy files (*.npy);; All files (*.*)")
        if len(_file[0]) > 0:
            self.ldtfrom.setText(_file[0])



    def clickBtnRtrvSeisProp(self):
        self.refreshMsgBox()
        #
        _attriblist = self.lwgattrib.selectedItems()
        if len(_attriblist) < 1:
            vis_msg.print("ERROR in RtrvSeisProb: No property selected for retrieval", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Retrieve Seismic Property',
                                           'No property selected for export')
            return
        if self.checkPointData() is False:
            vis_msg.print("ERROR in RtrvSeisProp: No point loaded for retrieval", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Retrieve Seismic Property',
                                           'No point loaded for retrieval')
            return
        #
        _wdinl = basic_data.str2int(self.ldtdimsinl.text())
        _wdxl = basic_data.str2int(self.ldtdimsxl.text())
        _wdz = basic_data.str2int(self.ldtdimsz.text())
        if _wdinl is False or _wdxl is False or _wdz is False:
            vis_msg.print("ERROR in RtrvSeisProp: Non-integer retrieval window", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Retrieve Seismic Property',
                                           'Non-integer retrieval window')
            return
        if _wdinl < 0 or _wdxl < 0 or _wdz < 0:
            vis_msg.print("ERROR in RtrvSeisProp: Non-positive retrieval window", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Retrieve Seismic Property',
                                           'Non-positive retrieval window')
            return
        #
        _wdsize = (2*_wdinl+1)*(2*_wdxl+1)*(2*_wdz+1)
        #
        # Progress dialog
        _pgsdlg = QtWidgets.QProgressDialog()
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/retrieve.png")),
                       QtGui.QIcon.Normal, QtGui.QIcon.Off)
        _pgsdlg.setWindowIcon(icon)
        _pgsdlg.setWindowTitle('Retrieve Seismic Property')
        _pgsdlg.setCancelButton(None)
        _pgsdlg.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
        _pgsdlg.forceShow()
        _pgsdlg.setFixedWidth(400)
        #
        _targetdata = basic_mdt.exportMatDict(self.pointsetdata, ['Inline', 'Crossline', 'Z'])
        #
        for i in range(len(_attriblist)):
            #
            _pgsdlg.setWindowTitle('Retrieve ' + str(i + 1) + ' of ' + str(len(_attriblist)) + ' Property')
            #
            print("RtrvSeisProp: Retrieve %d of %d Properties: %s" % (i + 1, len(_attriblist), _attriblist[i].text()))
            _npydata = self.npydata[_attriblist[i].text()]
            _data = seis_ays.retrieveSeisWindowFrom3DMat(_npydata, _targetdata, seisinfo=self.npyinfo,
                                                         wdinl=_wdinl, wdxl=_wdxl, wdz=_wdz,
                                                         verbose=False, qpgsdlg=_pgsdlg)
            _data = _data[:, 3:3+_wdsize]
            _data = np.mean(_data, axis=1)
            _data = np.reshape(_data, [-1, 1])
            self.pointsetdata[_attriblist[i].text()] = np.reshape(_data, [-1, 1])
            #
            # _pgsdlg.setValue(i + 1)
            #
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Retrieve Seismic Property",
                                          str(len(_attriblist)) + " properties retrieved successfully")
        return


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkPointData(self):
        return point_ays.checkPoint(self.pointsetdata)


def checkSeisData(seisdata, survinfo={}):
    return seis_ays.isSeis3DMatConsistentWithSeisInfo(seisdata, survinfo)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    RtrvSeisProp = QtWidgets.QWidget()
    gui = rtrvseisprop()
    gui.setupGUI(RtrvSeisProp)
    RtrvSeisProp.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     April 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for view segy trace header


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.seismic.inputoutput import inputoutput as seis_io

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class viewsegytraceheader(object):

    segyfile = ""
    #
    iconpath = os.path.dirname(__file__)
    dialog = None
    #
    tracenumlist = [10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000]


    def setupGUI(self, ViewSegyTraceHeader):
        ViewSegyTraceHeader.setObjectName("ViewSegyTraceHeader")
        ViewSegyTraceHeader.setFixedSize(850, 460)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/segy.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ViewSegyTraceHeader.setWindowIcon(icon)
        #
        self.lblhelp = QtWidgets.QLabel(ViewSegyTraceHeader)
        self.lblhelp.setObjectName("lblhelp")
        self.lblhelp.setGeometry(QtCore.QRect(750, 10, 80, 30))
        self.lbltracenum = QtWidgets.QLabel(ViewSegyTraceHeader)
        self.lbltracenum.setObjectName("lbltracenum")
        self.lbltracenum.setGeometry(QtCore.QRect(570, 10, 80, 30))
        self.cbbtracenum = QtWidgets.QComboBox(ViewSegyTraceHeader)
        self.cbbtracenum.setObjectName("cbbtracenum")
        self.cbbtracenum.setGeometry(QtCore.QRect(650, 10, 60, 30))
        self.twgheader = QtWidgets.QTableWidget(ViewSegyTraceHeader)
        self.twgheader.setObjectName("twgheader")
        self.twgheader.setGeometry(QtCore.QRect(10, 50, 820, 380))
        self.twgheader.setRowCount(88)
        self.twgheader.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        #
        self.msgbox = QtWidgets.QMessageBox(ViewSegyTraceHeader)
        self.msgbox.setObjectName("msgbox")
        _center_x = ViewSegyTraceHeader.geometry().center().x()
        _center_y = ViewSegyTraceHeader.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ViewSegyTraceHeader)
        QtCore.QMetaObject.connectSlotsByName(ViewSegyTraceHeader)



    def retranslateGUI(self, ViewSegyTraceHeader):
        self.dialog = ViewSegyTraceHeader
        #
        _translate = QtCore.QCoreApplication.translate
        ViewSegyTraceHeader.setWindowTitle(_translate("ViewSegyTraceHeader", "SEG-Y Trace Header"))
        self.lblhelp.setText(_translate("ViewSegyTraceHeader",
                                        '<a href="https://seg.org/Portals/0/SEG/News%20and%20Resources/Technical%20Standards/seg_y_rev1.pdf">SEG-Y Format</a>'))
        self.lblhelp.setOpenExternalLinks(True)
        #
        self.lbltracenum.setText(_translate("ViewSegyTraceHeader", "No. of traces:"))
        self.cbbtracenum.addItems([str(num) for num in self.tracenumlist])
        self.showTraceHeader(10)
        self.cbbtracenum.currentIndexChanged.connect(self.changeCbbTracenum)


    def changeCbbTracenum(self):
        self.showTraceHeader(self.tracenumlist[self.cbbtracenum.currentIndex()])


    def showTraceHeader(self, tracenum):
        self.twgheader.clear()
        #
        _rowheader = ['line_sequence_num (1-4)', 'file_sequence_num (5-8)',
                      'field_record_num (9-12)', 'trace_num (13-16)',
                      'energy_source_point_num (17-20)', 'ensemble_num (21-24)',
                      'ensemble_trace_num (25-28)', 'trace_identification_code (29-30)',
                      'num_vertically_summed_traces (31-32)', 'num_horizontally_stacked_traces (33-34)',
                      'data_use (35-36)', 'source_receiver_offset (37-40)',
                      'receiver_group_elevation (41-44)', 'surface_elevation_at_source (45-48)',
                      'source_depth_below_surface (49-52)', 'datum_elevation_at_receiver_group (53-56)',
                      'datum_elevation_at_source (57-60)', 'water_depth_at_source (61-64)',
                      'water_depth_at_group (65-68)', 'elevation_scalar (69-70)',
                      'xy_scalar (71-72)', 'source_x (73-76)',
                      'source_y (77-80)', 'group_x (81-84)',
                      'group_y (85-88)', 'coordinate_units (89-90)',
                      'weathering_velocity (91-92)', 'subweathering_velocity (93-94)',
                      'uphole_time_at_source (95-96)', 'uphole_time_at_group (97-98)',
                      'source_static_correction (99-100)', 'group_static_correction (101-102)',
                      'total_static (103-104)', 'lag_time_a (105-106)',
                      'lag_time_b (107-108)', 'delay_recording_time (109-110)',
                      'mute_start_time (111-112)', 'mute_end_time (113-114)',
                      'num_samples (115-116)', 'sample_interval (117-118)',
                      'gain_type_of_field_instruments (119-120)', 'instrument_gain_constant (121-122)',
                      'instrument_initial_gain (123-124)', 'correlated (125-126)',
                      'sweep_frequency_at_start (127-128)', 'sweep_frequency_at_end (129-130)',
                      'sweep_length (131-132)', 'sweep_type (133-134)',
                      'sweep_trace_taper_length_at_start (135-136)', 'sweep_trace_taper_length_at_end (137-138)',
                      'taper_type (139-140)', 'alias_filter_frequency (141-142)',
                      'alias_filter_slope (143-144)', 'notch_filter_frequency (145-146)',
                      'notch_filter_slope (147-148)', 'low_cut_frequency (149-150)',
                      'high_cut_frequency (151-152)', 'low_cut_slope (153-154)',
                      'high_cut_slope (155-156)', 'year_recorded (157-158)',
                      'day_of_year (159-160)', 'hour_of_day (161-162)',
                      'minute_of_hour (163-164)', 'second_of_minute (165-166)',
                      'time_basis_code (167-168)', 'trace_weighting_factor (169-170)',
                      'geophone_group_num_roll_switch_position_one (171-172)',
                      'geophone_group_num_first_trace_original_field (173-174)',
                      'geophone_group_num_last_trace_original_field (175-176)', 'gap_size (177-178)',
                      'over_travel (179-180)', 'cdp_x (181-184)',
                      'cdp_y (185-189)', 'inline_number (189-192)',
                      'crossline_number (193-196)', 'shotpoint_number (197-200)',
                      'shotpoint_scalar (201-202)', 'trace_unit (203-204)',
                      'transduction_constant_mantissa (205-208)', 'transduction_constant_exponent (208-210)',
                      'transduction_units (211-212)', 'device_trace_identifier (213-214)',
                      'time_scalar (215-216)', 'source_type (217-218)',
                      'source_energy_direction (219-224)', 'source_measurement_mantissa (225-228)',
                      'source_measurement_exponent (229-230)', 'source_measurement_unit (231-232)']
        self.twgheader.setVerticalHeaderLabels(_rowheader)
        #
        if os.path.exists(self.segyfile):
            _traceidx = np.linspace(0, tracenum-1, tracenum, dtype=int)
            self.twgheader.setColumnCount(tracenum)
            _colheader = ["Trace "+str(i+1) for i in _traceidx]
            self.twgheader.setHorizontalHeaderLabels(_colheader)
            #
            # Progress dialog
            _pgsdlg = QtWidgets.QProgressDialog()
            icon = QtGui.QIcon()
            icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/segy.png")),
                           QtGui.QIcon.Normal, QtGui.QIcon.Off)
            _pgsdlg.setWindowIcon(icon)
            _pgsdlg.setWindowTitle('View SEG-Y Trace Header')
            _pgsdlg.setCancelButton(None)
            _pgsdlg.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
            _pgsdlg.forceShow()
            _pgsdlg.setFixedWidth(400)
            _pgsdlg.setMaximum(tracenum)
            #
            for _idx in _traceidx:
                #
                QtCore.QCoreApplication.instance().processEvents()
                _pgsdlg.setValue(_idx+1)
                #
                _header = seis_io.readSegyTraceHeader(self.segyfile, traceidx=_idx)
                # line_dequence_num
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.line_sequence_num))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(0, _idx, _item)
                # file_sequence_num
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.file_sequence_num))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(1, _idx, _item)
                # field_record_num
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.field_record_num))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(2, _idx, _item)
                # trace_num
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.trace_num))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(3, _idx, _item)
                # energy_source_point_num
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.energy_source_point_num))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(4, _idx, _item)
                # ensemble_num
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.ensemble_num))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(5, _idx, _item)
                # ensemble_trace_num
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.ensemble_trace_num))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(6, _idx, _item)
                # trace_identification_code
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.trace_identification_code))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(7, _idx, _item)
                # num_vertically_summed_traces
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.num_vertically_summed_traces))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(8, _idx, _item)
                # num_horizontally_stacked_traces
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.num_horizontally_stacked_traces))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(9, _idx, _item)
                # data_use
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.data_use))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(10, _idx, _item)
                # source_receiver_offset
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.source_receiver_offset))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(11, _idx, _item)
                # receiver_group_elevation
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.receiver_group_elevation))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(12, _idx, _item)
                # surface_elevation_at_source
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.surface_elevation_at_source))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(13, _idx, _item)
                # source_depth_below_surface
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.source_depth_below_surface))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(14, _idx, _item)
                # datum_elevation_at_receiver_group
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.datum_elevation_at_receiver_group))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(15, _idx, _item)
                # datum_elevation_at_source
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.datum_elevation_at_source))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(16, _idx, _item)
                # water_depth_at_source
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.water_depth_at_source))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(17, _idx, _item)
                # water_depth_at_group
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.water_depth_at_group))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(18, _idx, _item)
                # elevation_scalar
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.elevation_scalar))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(19, _idx, _item)
                # xy_scalar
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.xy_scalar))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(20, _idx, _item)
                # source_x
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.source_x))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(21, _idx, _item)
                # source_y
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.source_y))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(22, _idx, _item)
                # group_x
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.group_x))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(23, _idx, _item)
                # group_y
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.group_y))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(24, _idx, _item)
                # coordinate_units
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.coordinate_units))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(25, _idx, _item)
                # weathering_velocity
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.weathering_velocity))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(26, _idx, _item)
                # subweathering_velocity
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.subweathering_velocity))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(27, _idx, _item)
                # uphole_time_at_source
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.uphole_time_at_source))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(28, _idx, _item)
                # uphole_time_at_group
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.uphole_time_at_group))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(29, _idx, _item)
                # source_static_correction
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.source_static_correction))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(30, _idx, _item)
                # group_static_correction
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.group_static_correction))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(31, _idx, _item)
                # total_static
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.total_static))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(32, _idx, _item)
                # lag_time_a
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.lag_time_a))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(33, _idx, _item)
                # lag_time_b
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.lag_time_b))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(34, _idx, _item)
                # delay_recording_time
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.delay_recording_time))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(35, _idx, _item)
                # mute_start_time
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.mute_start_time))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(36, _idx, _item)
                # mute_end_time
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.mute_end_time))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(37, _idx, _item)
                # num_samples
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.num_samples))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(38, _idx, _item)
                # sample_interval
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.sample_interval))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(39, _idx, _item)
                # gain_type_of_field_instruments
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.gain_type_of_field_instruments))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(40, _idx, _item)
                # instrument_gain_constant
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.instrument_gain_constant))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(41, _idx, _item)
                # instrument_initial_gain
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.instrument_initial_gain))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(42, _idx, _item)
                # correlated
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.correlated))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(43, _idx, _item)
                # sweep_frequency_at_start
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.sweep_frequency_at_start))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(44, _idx, _item)
                # sweep_frequency_at_end
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.sweep_frequency_at_end))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(45, _idx, _item)
                # sweep_length
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.sweep_length))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(46, _idx, _item)
                # sweep_type
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.sweep_type))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(47, _idx, _item)
                # sweep_trace_taper_length_at_start
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.sweep_trace_taper_length_at_start))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(48, _idx, _item)
                # sweep_trace_taper_length_at_end
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.sweep_trace_taper_length_at_end))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(49, _idx, _item)
                # taper_type
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.taper_type))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(50, _idx, _item)
                # alias_filter_frequency
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.alias_filter_frequency))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(51, _idx, _item)
                # alias_filter_slope
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.alias_filter_slope))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(52, _idx, _item)
                # notch_filter_frequency
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.notch_filter_frequency))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(53, _idx, _item)
                # notch_filter_slope
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.notch_filter_slope))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(54, _idx, _item)
                # low_cut_frequency
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.low_cut_frequency))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(55, _idx, _item)
                # high_cut_frequency
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.high_cut_frequency))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(56, _idx, _item)
                # low_cut_slope
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.low_cut_slope))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(57, _idx, _item)
                # high_cut_slope
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.high_cut_slope))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(58, _idx, _item)
                # year_recorded
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.year_recorded))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(59, _idx, _item)
                # day_of_year
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.day_of_year))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(60, _idx, _item)
                # hour_of_day
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.hour_of_day))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(61, _idx, _item)
                # minute_of_hour
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.minute_of_hour))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(62, _idx, _item)
                # second_of_minute
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.second_of_minute))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(63, _idx, _item)
                # time_basis_code
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.time_basis_code))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(64, _idx, _item)
                # trace_weighting_factor
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.trace_weighting_factor))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(65, _idx, _item)
                # geophone_group_num_roll_switch_position_one
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.geophone_group_num_roll_switch_position_one))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(66, _idx, _item)
                # geophone_group_num_first_trace_original_field
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(
                    str(_header.geophone_group_num_first_trace_original_field))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(67, _idx, _item)
                # geophone_group_num_last_trace_original_field
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(
                    str(_header.geophone_group_num_last_trace_original_field))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(68, _idx, _item)
                # gap_size
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.gap_size))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(69, _idx, _item)
                # over_travel
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.over_travel))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(70, _idx, _item)
                # cdp_x
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.cdp_x))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(71, _idx, _item)
                # cdp_y
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.cdp_y))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(72, _idx, _item)
                # inline_number
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.inline_number))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(73, _idx, _item)
                # crossline_number
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.crossline_number))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(74, _idx, _item)
                # shotpoint_number
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.shotpoint_number))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(75, _idx, _item)
                # shotpoint_scalar
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.shotpoint_scalar))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(76, _idx, _item)
                # trace_unit
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.trace_unit))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(77, _idx, _item)
                # transduction_constant_mantissa
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.transduction_constant_mantissa))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(78, _idx, _item)
                # transduction_constant_exponent
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.transduction_constant_exponent))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(79, _idx, _item)
                # transduction_units
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.transduction_units))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(80, _idx, _item)
                # device_trace_identifier
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.device_trace_identifier))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(81, _idx, _item)
                # time_scalar
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.time_scalar))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(82, _idx, _item)
                # source_type
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.source_type))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(83, _idx, _item)
                # source_energy_direction
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.source_energy_direction))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(84, _idx, _item)
                # source_measurement_mantissa
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.source_measurement_mantissa))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(85, _idx, _item)
                # source_measurement_exponent
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.source_measurement_exponent))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(86, _idx, _item)
                # source_measurement_unit
                _item = QtWidgets.QTableWidgetItem()
                _item.setText(str(_header.source_measurement_unit))
                _item.setFlags(QtCore.Qt.ItemIsEditable)
                _item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgheader.setItem(87, _idx, _item)
            #
            _pgsdlg.setValue(tracenum)


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ViewSegyTraceHeader = QtWidgets.QWidget()
    gui = viewsegytraceheader()
    gui.setupGUI(ViewSegyTraceHeader)
    ViewSegyTraceHeader.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# Create a window for view pre-stack seismic


from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
import numpy as np
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.core.settings import settings as core_set
from cognitivegeo.src.psseismic.analysis import analysis as psseis_ays
from cognitivegeo.src.gui.plotvis2dpsseisshot import plotvis2dpsseisshot as gui_plotvis2dpsseisshot

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class viewpsseis(object):

    psseisname = ''
    psseisdata = {}
    plotstyle = core_set.Visual['Image']
    fontstyle = core_set.Visual['Font']
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ViewPsSeis):
        ViewPsSeis.setObjectName("ViewPsSeis")
        ViewPsSeis.setFixedSize(510, 460)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/psseismic.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ViewPsSeis.setWindowIcon(icon)
        #
        self.btnplot = QtWidgets.QPushButton(ViewPsSeis)
        self.btnplot.setObjectName("btnplot")
        self.btnplot.setGeometry(QtCore.QRect(410, 10, 80, 30))
        self.btnplot.setEnabled(False)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/gather.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnplot.setIcon(icon)
        self.twgpsseis = QtWidgets.QTableWidget(ViewPsSeis)
        self.twgpsseis.setObjectName("twgpsseis")
        self.twgpsseis.setGeometry(QtCore.QRect(10, 50, 480, 380))
        self.twgpsseis.setColumnCount(3)
        self.twgpsseis.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        #
        self.msgbox = QtWidgets.QMessageBox(ViewPsSeis)
        self.msgbox.setObjectName("msgbox")
        _center_x = ViewPsSeis.geometry().center().x()
        _center_y = ViewPsSeis.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ViewPsSeis)
        QtCore.QMetaObject.connectSlotsByName(ViewPsSeis)



    def retranslateGUI(self, ViewPsSeis):
        self.dialog = ViewPsSeis
        #
        _translate = QtCore.QCoreApplication.translate
        ViewPsSeis.setWindowTitle(_translate("ViewPsSeis", "View Pre-stack Seismic " +self.psseisname))
        self.btnplot.setText(_translate("ViewPsSeis", "Plot"))
        self.btnplot.clicked.connect(self.clickBtnPlot)
        self.twgpsseis.setColumnCount(10)
        self.twgpsseis.setHorizontalHeaderLabels(["Shot",
                                                  "Trace No. per line", "Line No.",
                                                  "Null Trace No.",
                                                  "Sample No.", "Sample Interval",
                                                  "Maximum", "Minimum", "Mean", "Std"])
        if self.checkPsSeis():
            # self.btncopy.setEnabled(True)
            self.btnplot.setEnabled(True)
            #
            _nrow = len(list(self.psseisdata.keys()))
            self.twgpsseis.setRowCount(_nrow)
            self.twgpsseis.verticalHeader().hide()
            _idx = 0
            for i in sorted(self.psseisdata.keys()):
                item = QtWidgets.QTableWidgetItem()
                item.setText(_translate("ViewPsSeis", i))
                item.setFlags(QtCore.Qt.ItemIsEditable)
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgpsseis.setItem(_idx, 0, item)
                #
                item = QtWidgets.QTableWidgetItem()
                item.setText(_translate("ViewPsSeis", str(self.psseisdata[i]['ShotInfo']['XLNum'])))
                item.setFlags(QtCore.Qt.ItemIsEditable)
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgpsseis.setItem(_idx, 1, item)
                #
                item = QtWidgets.QTableWidgetItem()
                item.setText(_translate("ViewPsSeis", str(self.psseisdata[i]['ShotInfo']['ILNum'])))
                item.setFlags(QtCore.Qt.ItemIsEditable)
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgpsseis.setItem(_idx, 2, item)
                #
                item = QtWidgets.QTableWidgetItem()
                item.setText(_translate("ViewPsSeis",
                                        str(np.sum(self.psseisdata[i]['ShotInfo']['TraceFlag']).astype(int))))
                item.setFlags(QtCore.Qt.ItemIsEditable)
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgpsseis.setItem(_idx, 3, item)
                #
                item = QtWidgets.QTableWidgetItem()
                item.setText(_translate("ViewPsSeis", str(self.psseisdata[i]['ShotInfo']['ZNum'])))
                item.setFlags(QtCore.Qt.ItemIsEditable)
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgpsseis.setItem(_idx, 4, item)
                #
                item = QtWidgets.QTableWidgetItem()
                item.setText(_translate("ViewPsSeis", str(self.psseisdata[i]['ShotInfo']['ZStep'])))
                item.setFlags(QtCore.Qt.ItemIsEditable)
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgpsseis.setItem(_idx, 5, item)
                #
                _data = self.psseisdata[i]['ShotData']
                #
                item = QtWidgets.QTableWidgetItem()
                item.setText(_translate("ViewPsSeis", str(np.max(_data))))
                item.setFlags(QtCore.Qt.ItemIsEditable)
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgpsseis.setItem(_idx, 6, item)
                #
                item = QtWidgets.QTableWidgetItem()
                item.setText(_translate("ViewPsSeis", str(np.min(_data))))
                item.setFlags(QtCore.Qt.ItemIsEditable)
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgpsseis.setItem(_idx, 7, item)
                #
                item = QtWidgets.QTableWidgetItem()
                item.setText(_translate("ViewPsSeis", str(np.mean(_data))))
                item.setFlags(QtCore.Qt.ItemIsEditable)
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgpsseis.setItem(_idx, 8, item)
                #
                item = QtWidgets.QTableWidgetItem()
                item.setText(_translate("ViewPsSeis", str(np.std(_data))))
                item.setFlags(QtCore.Qt.ItemIsEditable)
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.twgpsseis.setItem(_idx, 9, item)
                #
                _idx = _idx + 1


    def clickBtnPlot(self):
        _plt = QtWidgets.QDialog()
        _gui = gui_plotvis2dpsseisshot()
        _gui.psseisdata = {}
        _gui.psseisdata[self.psseisname] = self.psseisdata
        _gui.plotstyle = self.plotstyle
        _gui.fontstyle = self.fontstyle
        _gui.setupGUI(_plt)
        _plt.exec()
        _plt.show()


    def checkPsSeis(self):
        return psseis_ays.checkPsSeis(self.psseisdata)


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ViewPsSeis = QtWidgets.QWidget()
    gui = viewpsseis()
    gui.setupGUI(ViewPsSeis)
    ViewPsSeis.show()
    sys.exit(app.exec_())